@@ -33,7 +33,6 @@
 
 #include <stdio.h>
 #include <errno.h>
-#include <assert.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -41,6 +40,7 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#include "lnav_log.hh"
 #include "grep_proc.hh"
 
 #include "time_T.hh"
@@ -63,7 +63,7 @@ grep_proc::grep_proc(pcre *code,
       gp_sink(NULL),
       gp_control(NULL)
 {
-    assert(this->invariant());
+    require(this->invariant());
 }
 
 grep_proc::~grep_proc()
@@ -100,7 +100,7 @@ void grep_proc::handle_match(int line,
 
 void grep_proc::start(void)
 {
-    assert(this->invariant());
+    require(this->invariant());
 
     if (this->gp_child_started || this->gp_queue.empty()) {
         return;
@@ -134,7 +134,7 @@ void grep_proc::start(void)
 
         fcntl(err_pipe.read_end(), F_SETFL, O_NONBLOCK);
         fcntl(err_pipe.read_end(), F_SETFD, 1);
-        assert(this->gp_err_pipe.get() == -1);
+        require(this->gp_err_pipe.get() == -1);
         this->gp_err_pipe      = err_pipe.read_end();
         this->gp_child_started = true;
 
@@ -160,7 +160,7 @@ void grep_proc::start(void)
 
     this->child_loop();
 
-    exit(0);
+    _exit(0);
 }
 
 void grep_proc::child_loop(void)
@@ -244,7 +244,7 @@ void grep_proc::cleanup(void)
         while (waitpid(this->gp_child, &status, 0) < 0 && (errno == EINTR)) {
             ;
         }
-        assert(!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT);
+        require(!WIFSIGNALED(status) || WTERMSIG(status) != SIGABRT);
         this->gp_child         = -1;
         this->gp_child_started = false;
 
@@ -265,7 +265,7 @@ void grep_proc::cleanup(void)
     this->gp_pipe_offset = 0;
     this->gp_line_buffer.reset();
 
-    assert(this->invariant());
+    ensure(this->invariant());
 
     if (!this->gp_queue.empty()) {
         this->start();
@@ -276,7 +276,7 @@ void grep_proc::dispatch_line(char *line)
 {
     int start, end, capture_start;
 
-    assert(line != NULL);
+    require(line != NULL);
 
     if (sscanf(line, "%d", this->gp_last_line.out()) == 1) {
         /* Starting a new line with matches. */
@@ -284,11 +284,11 @@ void grep_proc::dispatch_line(char *line)
         if (this->gp_last_line > this->gp_highest_line) {
             this->gp_highest_line = this->gp_last_line;
         }
-        assert(this->gp_last_line >= 0);
+        ensure(this->gp_last_line >= 0);
     }
     else if (sscanf(line, "[%d:%d]", &start, &end) == 2) {
-        assert(start >= 0);
-        assert(end >= 0);
+        require(start >= 0);
+        require(end >= 0);
 
         /* Pass the match offsets to the sink delegate. */
         if (this->gp_sink != NULL) {
@@ -296,8 +296,8 @@ void grep_proc::dispatch_line(char *line)
         }
     }
     else if (sscanf(line, "(%d:%d)%n", &start, &end, &capture_start) == 2) {
-        assert(start == -1 || start >= 0);
-        assert(end >= 0);
+        require(start == -1 || start >= 0);
+        require(end >= 0);
 
         /* Pass the captured strings to the sink delegate. */
         if (this->gp_sink != NULL) {
@@ -315,13 +315,13 @@ void grep_proc::dispatch_line(char *line)
         }
     }
     else {
-        fprintf(stderr, "bad line from child -- %s\n", line);
+        log_error("bad line from child -- %s", line);
     }
 }
 
 void grep_proc::check_fd_set(fd_set &ready_fds)
 {
-    assert(this->invariant());
+    require(this->invariant());
 
     if (this->gp_err_pipe != -1 && FD_ISSET(this->gp_err_pipe, &ready_fds)) {
         char buffer[1024 + 1];
@@ -355,14 +355,12 @@ void grep_proc::check_fd_set(fd_set &ready_fds)
             static const int MAX_LOOPS = 100;
 
             int    loop_count = 0;
-            size_t len;
-            char * line;
+            line_value lv;
 
             while ((loop_count < MAX_LOOPS) &&
-                   (line = this->gp_line_buffer.read_line(this->gp_pipe_offset,
-                                                          len)) != NULL) {
-                line[len] = '\0';
-                this->dispatch_line(line);
+                   (this->gp_line_buffer.read_line(this->gp_pipe_offset, lv))) {
+                lv.lv_start[lv.lv_len] = '\0';
+                this->dispatch_line(lv.lv_start);
                 loop_count += 1;
             }
 
@@ -380,5 +378,5 @@ void grep_proc::check_fd_set(fd_set &ready_fds)
         }
     }
 
-    assert(this->invariant());
+    ensure(this->invariant());
 }
