@@ -35,10 +35,13 @@
 #include <math.h>
 #include <sys/time.h>
 
+#include "lnav_log.hh"
 #include "listview_curses.hh"
 
 using namespace std;
 
+list_gutter_source listview_curses::DEFAULT_GUTTER_SOURCE;
+
 listview_curses::listview_curses()
     : lv_source(NULL),
       lv_overlay_source(NULL),
@@ -49,7 +52,11 @@ listview_curses::listview_curses()
       lv_height(0),
       lv_needs_update(true),
       lv_show_scrollbar(true),
+      lv_show_bottom_border(false),
+      lv_gutter_source(&DEFAULT_GUTTER_SOURCE),
       lv_word_wrap(false),
+      lv_scroll_accel(0),
+      lv_scroll_velo(0),
       lv_mouse_y(-1),
       lv_mouse_mode(LV_MODE_NONE)
 { }
@@ -111,10 +118,12 @@ bool listview_curses::handle_key(int ch)
         this->shift_top(this->rows_available(this->lv_top, RD_DOWN) - vis_line_t(1));
         break;
 
+    case 'g':
     case KEY_HOME:
         this->set_top(vis_line_t(0));
         break;
 
+    case 'G':
     case KEY_END: {
             vis_line_t last_line(this->get_inner_height() - 1);
             vis_line_t tail_bottom(this->get_top_for_last_row());
@@ -168,7 +177,8 @@ void listview_curses::do_update(void)
         }
 
         this->get_dimensions(height, width);
-        wrap_width = width - (this->lv_word_wrap ? 1 : 0);
+
+        wrap_width = width - (this->lv_word_wrap ? 1 : this->lv_show_scrollbar ? 1 : 0);
 
         row_count = this->get_inner_height();
         row   = this->lv_top;
@@ -211,9 +221,10 @@ void listview_curses::do_update(void)
         if (this->lv_show_scrollbar) {
             double progress = 1.0;
             double coverage = 1.0;
+            double adjusted_height = (double)row_count / (double)height;
             vis_line_t lines;
 
-            if (this->get_inner_height() > 0) {
+            if (row_count > 0) {
                 progress = (double)this->lv_top / (double)row_count;
                 coverage = (double)height / (double)row_count;
             }
@@ -222,17 +233,40 @@ void listview_curses::do_update(void)
                 vis_line_t((int)(progress * (double)height));
             lines = y + min(height, vis_line_t(
                                 (int)(coverage * (double)height)));
-            for (; y <= lines; ++y) {
-                mvwchgat(this->lv_window,
-                         y, width - 1,
-                         1,
-                         A_REVERSE, view_colors::ansi_color_pair(
-                            COLOR_WHITE, COLOR_BLACK),
-                         NULL);
+
+            for (int gutter_y = this->lv_y;
+                 gutter_y < (this->lv_y + height);
+                 gutter_y++) {
+                int range_start = 0, range_end;
+                int fg = COLOR_WHITE, bg = COLOR_BLACK, attrs;
+                chtype ch = ACS_VLINE;
+
+                if (row_count > 0) {
+                    range_start = (double)(gutter_y - this->lv_y) * adjusted_height;
+                }
+                range_end = range_start + adjusted_height;
+
+                this->lv_gutter_source->listview_gutter_value_for_range(
+                    *this, range_start, range_end, ch, fg);
+                if (gutter_y >= y && gutter_y <= lines) {
+                    bg = COLOR_WHITE;
+                    if (fg == bg) {
+                        fg = COLOR_BLACK;
+                    }
+                }
+                attrs = view_colors::ansi_color_pair(fg, bg);
+                wattron(this->lv_window, attrs);
+                mvwaddch(this->lv_window, gutter_y, width - 1, ch);
+                wattroff(this->lv_window, attrs);
             }
             wmove(this->lv_window, this->lv_y + height - 1, 0);
         }
 
+        if (this->lv_show_bottom_border) {
+            mvwchgat(this->lv_window,
+                this->lv_y + height - 1, 0, width - 1, A_UNDERLINE, 0, NULL);
+        }
+
         this->lv_needs_update = false;
     }
 }
@@ -260,7 +294,7 @@ bool listview_curses::handle_mouse(mouse_event &me)
             this->lv_scroll_velo = 0;
         }
         else {
-            this->lv_scroll_accel += 3;
+            this->lv_scroll_accel += 2;
         }
         this->lv_scroll_velo += this->lv_scroll_accel;
 
@@ -294,7 +328,7 @@ bool listview_curses::handle_mouse(mouse_event &me)
     scroll_bottom = (this->get_y() + (int)(bot_pct * (double)height));
 
     if (this->lv_mouse_mode == LV_MODE_NONE) {
-        if (scroll_top <= me.me_y && me.me_y <= scroll_bottom) {
+        if ((scroll_top - 1) <= me.me_y && me.me_y <= (scroll_bottom + 1)) {
             this->lv_mouse_mode        = LV_MODE_DRAG;
             this->lv_mouse_y = me.me_y - scroll_top;
         }
@@ -308,7 +342,7 @@ bool listview_curses::handle_mouse(mouse_event &me)
 
     switch (this->lv_mouse_mode) {
     case LV_MODE_NONE:
-        assert(0);
+        require(0);
         break;
 
     case LV_MODE_UP:
