@@ -44,9 +44,40 @@
 #include "auto_mem.hh"
 #include "log_data_table.hh"
 #include "lnav_commands.hh"
+#include "session_data.hh"
 
 using namespace std;
 
+static bool wordexperr(int rc, string &msg)
+{
+    switch (rc) {
+    case WRDE_BADCHAR:
+        msg = "error: invalid filename character";
+        return false;
+
+    case WRDE_CMDSUB:
+        msg = "error: command substitution is not allowed";
+        return false;
+
+    case WRDE_BADVAL:
+        msg = "error: unknown environment variable in file name";
+        return false;
+
+    case WRDE_NOSPACE:
+        msg = "error: out of memory";
+        return false;
+
+    case WRDE_SYNTAX:
+        msg = "error: invalid syntax";
+        return false;
+
+    default:
+        break;
+    }
+    
+    return true;
+}
+
 static string com_adjust_log_time(string cmdline, vector<string> &args)
 {
     string retval = "error: expecting new time value";
@@ -76,7 +107,7 @@ static string com_adjust_log_time(string cmdline, vector<string> &args)
         top_time = ll.get_timeval();
 
         dts.set_base_time(top_time.tv_sec);
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         if (dts.scan(args[1].c_str(), NULL, &tm, new_time) != NULL) {
             timersub(&new_time, &top_time, &time_diff);
             
@@ -109,7 +140,7 @@ static string com_unix_time(string cmdline, vector<string> &args)
 
         log_time.tm_isdst = -1;
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         if ((millis = args[1].find('.')) != string::npos ||
             (millis = args[1].find(',')) != string::npos) {
             args[1] = args[1].erase(millis, 4);
@@ -171,15 +202,31 @@ static string com_current_time(string cmdline, vector<string> &args)
 
 static string com_goto(string cmdline, vector<string> &args)
 {
-    string retval = "error: expecting line number/percentage";
+    string retval = "error: expecting line number/percentage or timestamp";
 
-    if (args.size() == 0) { }
+    if (args.size() == 0) {
+        args.push_back("line-time");
+    }
     else if (args.size() > 1) {
         textview_curses *tc = lnav_data.ld_view_stack.top();
         int   line_number, consumed;
+        date_time_scanner dts;
+        struct timeval tv;
+        struct tm tm;
         float value;
 
-        if (sscanf(args[1].c_str(), "%f%n", &value, &consumed) == 1) {
+        if (dts.scan(args[1].c_str(), NULL, &tm, tv) != NULL) {
+            if (tc == &lnav_data.ld_views[LNV_LOG]) {
+                vis_line_t vl;
+
+                vl = lnav_data.ld_log_source.find_from_time(tv);
+                tc->set_top(vl);
+                retval = "";
+            } else {
+                retval = "error: time values only work in the log view";
+            }
+        }
+        else if (sscanf(args[1].c_str(), "%f%n", &value, &consumed) == 1) {
             if (args[1][consumed] == '%') {
                 line_number = (int)
                               ((double)tc->get_inner_height() *
@@ -187,6 +234,9 @@ static string com_goto(string cmdline, vector<string> &args)
             }
             else {
                 line_number = (int)value;
+                if (line_number < 0) {
+                    line_number = tc->get_inner_height() + line_number;
+                }
             }
             tc->set_top(vis_line_t(line_number));
 
@@ -227,41 +277,62 @@ static void csv_write_string(FILE *outfile, const string &str)
     }
 }
 
+static void yajl_writer(void *context, const char *str, size_t len)
+{
+    FILE *file = (FILE *)context;
+
+    fwrite(str, len, 1, file);
+}
+
+static void json_write_row(yajl_gen handle, int row)
+{
+    db_label_source &dls = lnav_data.ld_db_rows;
+    yajlpp_map obj_map(handle);
+
+    for (int col = 0; col < dls.dls_column_types.size(); col++) {
+        obj_map.gen(dls.dls_headers[col]);
+
+        if (dls.dls_rows[row][col] == db_label_source::NULL_STR) {
+            obj_map.gen();
+            continue;
+        }
+
+        switch (dls.dls_column_types[col]) {
+        case SQLITE_FLOAT:
+        case SQLITE_INTEGER:
+            yajl_gen_number(handle, dls.dls_rows[row][col],
+                strlen(dls.dls_rows[row][col]));
+            break;
+        default:
+            obj_map.gen(dls.dls_rows[row][col]);
+            break;
+        }
+    }
+}
+
 static string com_save_to(string cmdline, vector<string> &args)
 {
     FILE *      outfile = NULL;
     const char *mode    = "";
+    string fn, retval;
 
     if (args.size() == 0) {
         args.push_back("filename");
         return "";
     }
 
-    if (args.size() != 2) {
+    if (args.size() < 2) {
         return "error: expecting file name";
     }
 
+    fn = trim(cmdline.substr(cmdline.find(args[1], args[0].size())));
+
     static_root_mem<wordexp_t, wordfree> wordmem;
 
-    switch (wordexp(args[1].c_str(), wordmem.inout(), WRDE_NOCMD |
-                    WRDE_UNDEF)) {
-    case WRDE_BADCHAR:
-        return "error: invalid filename character";
+    int rc = wordexp(fn.c_str(), wordmem.inout(), WRDE_NOCMD | WRDE_UNDEF);
 
-    case WRDE_CMDSUB:
-        return "error: command substitution is not allowed";
-
-    case WRDE_BADVAL:
-        return "error: unknown environment variable in file name";
-
-    case WRDE_NOSPACE:
-        return "error: out of memory";
-
-    case WRDE_SYNTAX:
-        return "error: invalid syntax";
-
-    default:
-        break;
+    if (!wordexperr(rc, retval)) {
+        return retval;
     }
 
     if (wordmem->we_wordc > 1) {
@@ -271,7 +342,7 @@ static string com_save_to(string cmdline, vector<string> &args)
     if (args[0] == "append-to") {
         mode = "a";
     }
-    else if (args[0] == "write-to" || args[0] == "write-csv-to") {
+    else {
         mode = "w";
     }
 
@@ -281,7 +352,7 @@ static string com_save_to(string cmdline, vector<string> &args)
         tc->get_bookmarks()[&textview_curses::BM_USER];
     db_label_source &dls = lnav_data.ld_db_rows;
 
-    if (args[0] == "write-csv-to") {
+    if (args[0] == "write-csv-to" || args[0] == "write-json-to") {
         if (dls.dls_headers.empty()) {
             return "error: no query result to write, use ';' to execute a query";
         }
@@ -292,22 +363,30 @@ static string com_save_to(string cmdline, vector<string> &args)
         }
     }
 
-    if ((outfile = fopen(wordmem->we_wordv[0], mode)) == NULL) {
+    if (strcmp(wordmem->we_wordv[0], "-") == 0) {
+        if (!(lnav_data.ld_flags & LNF_HEADLESS)) {
+            return "error: writing to stdout is only available in headless mode";
+        }
+        outfile = stdout;
+        lnav_data.ld_stdout_used = true;
+    }
+    else if ((outfile = fopen(wordmem->we_wordv[0], mode)) == NULL) {
         return "error: unable to open file -- " + string(wordmem->we_wordv[0]);
     }
 
     if (args[0] == "write-csv-to") {
-        std::vector<std::vector<string> >::iterator row_iter;
-        std::vector<string>::iterator iter;
+        std::vector<std::vector<const char *> >::iterator row_iter;
+        std::vector<const char *>::iterator iter;
+        std::vector<string>::iterator hdr_iter;
         bool first = true;
 
-        for (iter = dls.dls_headers.begin();
-             iter != dls.dls_headers.end();
-             ++iter) {
+        for (hdr_iter = dls.dls_headers.begin();
+             hdr_iter != dls.dls_headers.end();
+             ++hdr_iter) {
             if (!first) {
                 fprintf(outfile, ",");
             }
-            csv_write_string(outfile, *iter);
+            csv_write_string(outfile, *hdr_iter);
             first = false;
         }
         fprintf(outfile, "\n");
@@ -328,6 +407,26 @@ static string com_save_to(string cmdline, vector<string> &args)
             fprintf(outfile, "\n");
         }
     }
+    else if (args[0] == "write-json-to") {
+        yajl_gen handle = NULL;
+
+        if ((handle = yajl_gen_alloc(NULL)) == NULL) {
+            return "error: unable to allocate memory";
+        }
+        else {
+            yajl_gen_config(handle, yajl_gen_beautify, 1);
+            yajl_gen_config(handle,
+                yajl_gen_print_callback, yajl_writer, outfile);
+
+            {
+                yajlpp_array root_array(handle);
+
+                for (int row = 0; row < dls.dls_rows.size(); row++) {
+                    json_write_row(handle, row);
+                }
+            }
+        }
+    }
     else {
         bookmark_vector<vis_line_t>::iterator iter;
         string line;
@@ -338,7 +437,9 @@ static string com_save_to(string cmdline, vector<string> &args)
         }
     }
 
-    fclose(outfile);
+    if (outfile != stdout) {
+        fclose(outfile);
+    }
     outfile = NULL;
 
     return "";
@@ -356,7 +457,7 @@ static string com_highlight(string cmdline, vector<string> &args)
         pcre *      code;
         int         eoff;
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         if (hm.find(args[1]) != hm.end()) {
             retval = "error: highlight already exists";
         }
@@ -391,7 +492,7 @@ static string com_graph(string cmdline, vector<string> &args)
         pcre *      code;
         int         eoff;
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         if ((code = pcre_compile(args[1].c_str(),
                                  PCRE_CASELESS,
                                  &errptr,
@@ -445,34 +546,17 @@ static string com_help(string cmdline, vector<string> &args)
 class pcre_filter
     : public logfile_filter {
 public:
-    pcre_filter(type_t type, string id, pcre *code)
+    pcre_filter(type_t type, const string id, pcre *code)
         : logfile_filter(type, id),
-          pf_code(code) { };
+          pf_pcre(code) { };
     virtual ~pcre_filter() { };
 
-    bool matches(string line)
+    bool matches(const logline &ll, const string &line)
     {
-        static const int MATCH_COUNT = 20 * 3;
-        int  matches[MATCH_COUNT], rc;
-        bool retval;
+        pcre_context_static<30> pc;
+        pcre_input pi(line);
 
-        rc = pcre_exec(this->pf_code,
-                       NULL,
-                       line.c_str(),
-                       line.size(),
-                       0,
-                       0,
-                       matches,
-                       MATCH_COUNT);
-        retval = (rc >= 0);
-
-#if 0
-        fprintf(stderr, " out %d %s\n",
-                retval,
-                line.c_str());
-#endif
-
-        return retval;
+        return this->pf_pcre.match(pc, pi);
     };
 
     std::string to_command(void)
@@ -483,7 +567,7 @@ public:
     };
 
 protected:
-    auto_mem<pcre> pf_code;
+    pcrepp pf_pcre;
 };
 
 static string com_filter(string cmdline, vector<string> &args)
@@ -499,12 +583,12 @@ static string com_filter(string cmdline, vector<string> &args)
         pcre *      code;
         int         eoff;
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         if (lss.get_filter(args[1]) != NULL) {
             retval = "error: filter already exists";
         }
         else if ((code = pcre_compile(args[1].c_str(),
-                                      0,
+                                      PCRE_CASELESS,
                                       &errptr,
                                       &eoff,
                                       NULL)) == NULL) {
@@ -517,8 +601,10 @@ static string com_filter(string cmdline, vector<string> &args)
             auto_ptr<pcre_filter> pf(new pcre_filter(lt, args[1], code));
 
             lss.add_filter(pf.release());
-            lnav_data.ld_rl_view->
-            add_possibility(LNM_COMMAND, "enabled-filter", args[1]);
+            if (lnav_data.ld_rl_view != NULL) {
+                lnav_data.ld_rl_view->add_possibility(
+                    LNM_COMMAND, "enabled-filter", args[1]);
+            }
             rebuild_indexes(true);
 
             retval = "info: filter now active";
@@ -538,7 +624,7 @@ static string com_enable_filter(string cmdline, vector<string> &args)
     else if (args.size() > 1) {
         logfile_filter *lf;
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         lf      = lnav_data.ld_log_source.get_filter(args[1]);
         if (lf == NULL) {
             retval = "error: no such filter -- " + args[1];
@@ -548,10 +634,12 @@ static string com_enable_filter(string cmdline, vector<string> &args)
         }
         else {
             lnav_data.ld_log_source.set_filter_enabled(lf, true);
-            lnav_data.ld_rl_view->
-            rem_possibility(LNM_COMMAND, "disabled-filter", args[1]);
-            lnav_data.ld_rl_view->
-            add_possibility(LNM_COMMAND, "enabled-filter", args[1]);
+            if (lnav_data.ld_rl_view != NULL) {
+                lnav_data.ld_rl_view->rem_possibility(
+                    LNM_COMMAND, "disabled-filter", args[1]);
+                lnav_data.ld_rl_view->add_possibility(
+                    LNM_COMMAND, "enabled-filter", args[1]);
+            }
             rebuild_indexes(true);
             retval = "info: filter enabled";
         }
@@ -570,7 +658,7 @@ static string com_disable_filter(string cmdline, vector<string> &args)
     else if (args.size() > 1) {
         logfile_filter *lf;
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = cmdline.substr(cmdline.find(args[1], args[0].size()));
         lf      = lnav_data.ld_log_source.get_filter(args[1]);
         if (lf == NULL) {
             retval = "error: no such filter -- " + args[1];
@@ -580,10 +668,12 @@ static string com_disable_filter(string cmdline, vector<string> &args)
         }
         else {
             lnav_data.ld_log_source.set_filter_enabled(lf, false);
-            lnav_data.ld_rl_view->
-            rem_possibility(LNM_COMMAND, "disabled-filter", args[1]);
-            lnav_data.ld_rl_view->
-            add_possibility(LNM_COMMAND, "enabled-filter", args[1]);
+            if (lnav_data.ld_rl_view != NULL) {
+                lnav_data.ld_rl_view->rem_possibility(
+                    LNM_COMMAND, "disabled-filter", args[1]);
+                lnav_data.ld_rl_view->add_possibility(
+                    LNM_COMMAND, "enabled-filter", args[1]);
+            }
             rebuild_indexes(true);
             retval = "info: filter disabled";
         }
@@ -641,15 +731,17 @@ static string com_create_logline_table(string cmdline, vector<string> &args)
         }
         else {
             vis_line_t      vl  = log_view.get_top();
-            content_line_t  cl  = lnav_data.ld_log_source.at(vl);
+            content_line_t  cl  = lnav_data.ld_log_source.at_base(vl);
             log_data_table *ldt = new log_data_table(cl, args[1]);
             string          errmsg;
 
             errmsg = lnav_data.ld_vtab_manager->register_vtab(ldt);
             if (errmsg.empty()) {
-                lnav_data.ld_rl_view->add_possibility(LNM_COMMAND,
-                                                      "custom-table",
-                                                      args[1]);
+                if (lnav_data.ld_rl_view != NULL) {
+                    lnav_data.ld_rl_view->add_possibility(LNM_COMMAND,
+                      "custom-table",
+                      args[1]);
+                }
                 retval = "info: created new log table -- " + args[1];
             }
             else {
@@ -672,9 +764,11 @@ static string com_delete_logline_table(string cmdline, vector<string> &args)
         string rc = lnav_data.ld_vtab_manager->unregister_vtab(args[1]);
 
         if (rc.empty()) {
-            lnav_data.ld_rl_view->rem_possibility(LNM_COMMAND,
-                                                  "custom-table",
-                                                  args[1]);
+            if (lnav_data.ld_rl_view != NULL) {
+                lnav_data.ld_rl_view->rem_possibility(LNM_COMMAND,
+                  "custom-table",
+                  args[1]);
+            }
             retval = "info: deleted logline table";
         }
         else {
@@ -761,19 +855,32 @@ static string com_open(string cmdline, vector<string> &args)
 
     if (args.size() == 0) {
         args.push_back("filename");
+        return "";
+    }
+    else if (args.size() < 2) {
+        return retval;
     }
-    else if (args.size() > 1) {
-        list<logfile *>::iterator file_iter;
-        size_t colon_index;
-        int top = 0;
-        string fn;
 
-        fn = cmdline.substr(cmdline.find(args[1]));
+    static_root_mem<wordexp_t, wordfree> wordmem;
+    list<logfile *>::iterator file_iter;
+    size_t colon_index;
+    int top = 0;
+    string pat;
+
+    pat = trim(cmdline.substr(cmdline.find(args[1], args[0].size())));
+
+    int rc = wordexp(pat.c_str(), wordmem.inout(), WRDE_NOCMD | WRDE_UNDEF);
+
+    if (!wordexperr(rc, retval)) {
+        return retval;
+    }
+
+    for (int lpc = 0; lpc < wordmem->we_wordc; lpc++) {
+        string fn = wordmem->we_wordv[lpc];
 
         if (access(fn.c_str(), R_OK) != 0 &&
             (colon_index = fn.rfind(':')) != string::npos) {
-            if (sscanf(&fn.c_str()[colon_index + 1], "%d", &top) == 1 &&
-                top >= 0) {
+            if (sscanf(&fn.c_str()[colon_index + 1], "%d", &top) == 1) {
                 fn = fn.substr(0, colon_index);
             }
         }
@@ -804,16 +911,38 @@ static string com_open(string cmdline, vector<string> &args)
                 retval = "info: watching -- " + fn;
             }
             else if (stat(fn.c_str(), &st) == -1) {
-                retval = "error: cannot stat file: " + fn + " " + strerror(errno);
+                retval = ("error: cannot stat file: " + fn + " -- "
+                    + strerror(errno));
             }
             else if ((abspath = realpath(fn.c_str(), NULL)) == NULL) {
                 retval = "error: cannot find file";
             }
+            else if (S_ISDIR(st.st_mode)) {
+                string dir_wild(abspath.in());
+
+                if (dir_wild[dir_wild.size() - 1] == '/') {
+                    dir_wild.resize(dir_wild.size() - 1);
+                }
+                lnav_data.ld_file_names.insert(make_pair(dir_wild + "/*", -1));
+                retval = "info: watching -- " + dir_wild;
+            }
+            else if (!S_ISREG(st.st_mode)) {
+                retval = "error: not a regular file or directory";
+            }
+            else if (access(fn.c_str(), R_OK) == -1) {
+                retval = (string("error: cannot read file -- ") +
+                    strerror(errno));
+            }
             else {
                 fn = abspath.in();
                 lnav_data.ld_file_names.insert(make_pair(fn, -1));
                 retval = "info: opened -- " + fn;
                 lnav_data.ld_files_to_front.push_back(make_pair(fn, top));
+
+                if (lnav_data.ld_rl_view != NULL) {
+                    lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(
+                        X, "to close the file"));
+                }
             }
         }
     }
@@ -875,29 +1004,56 @@ static string com_partition_name(string cmdline, vector<string> &args)
     string retval = "error: expecting partition name";
 
     if (args.size() == 0) {
-        args.push_back("enabled-filter");
+        return "";
     }
     else if (args.size() > 1) {
         textview_curses &tc = lnav_data.ld_views[LNV_LOG];
         logfile_sub_source &lss = lnav_data.ld_log_source;
         std::map<content_line_t, bookmark_metadata> &bm = lss.get_user_bookmark_metadata();
 
-        args[1] = cmdline.substr(cmdline.find(args[1]));
+        args[1] = trim(cmdline.substr(cmdline.find(args[1], args[0].size())));
 
-        bookmark_vector<vis_line_t> &bv = tc.get_bookmarks()[&textview_curses::BM_USER];
-        bookmark_vector<vis_line_t>::iterator iter;
+        tc.set_user_mark(&textview_curses::BM_PARTITION, tc.get_top(), true);
 
-        iter = lower_bound(bv.begin(), bv.end(), vis_line_t(tc.get_top() + 1));
-        if (iter == bv.begin()) {
-            retval = "error: no marks, press 'm' to mark the start of a partition";
+        bookmark_metadata &line_meta = bm[lss.at(tc.get_top())];
+
+        line_meta.bm_name = args[1];
+        retval = "info: name set for partition";
+    }
+
+    return retval;
+}
+
+static string com_clear_partition(string cmdline, vector<string> &args)
+{
+    string retval = "";
+
+    if (args.size() == 0) {
+        return "";
+    }
+    else if (args.size() == 1) {
+        textview_curses &tc = lnav_data.ld_views[LNV_LOG];
+        logfile_sub_source &lss = lnav_data.ld_log_source;
+        bookmark_vector<vis_line_t> &bv = tc.get_bookmarks()[
+            &textview_curses::BM_PARTITION];
+        std::map<content_line_t, bookmark_metadata> &bm = lss.get_user_bookmark_metadata();
+        vis_line_t part_start;
+
+        if (binary_search(bv.begin(), bv.end(), tc.get_top())) {
+            part_start = tc.get_top();
         }
         else {
-            --iter;
+            part_start = bv.prev(tc.get_top());
+        }
+        if (part_start == -1) {
+            retval = "error: top line is not in a partition";
+        }
+        else {
+            tc.set_user_mark(
+                &textview_curses::BM_PARTITION, part_start, false);
 
-            bookmark_metadata &line_meta = bm[lss.at(*iter)];
-
-            line_meta.bm_name = args[1];
-            retval = "info: name set for partition";
+            bm.erase(lss.at(part_start));
+            retval = "info: cleared partition name";
         }
     }
 
@@ -941,6 +1097,12 @@ static string com_summarize(string cmdline, vector<string> &args)
                                      -1,
                                      stmt.out(),
                                      NULL);
+        if (retcode != SQLITE_OK) {
+            const char *errmsg = sqlite3_errmsg(lnav_data.ld_db);
+
+            return "error: " + string(errmsg);
+        }
+
         switch (sqlite3_step(stmt.in())) {
             case SQLITE_OK:
             case SQLITE_DONE:
@@ -1041,8 +1203,6 @@ static string com_summarize(string cmdline, vector<string> &args)
             query += query_frag;
         }
 
-        fprintf(stderr, "query %s\n", query.c_str());
-
         db_label_source &      dls = lnav_data.ld_db_rows;
         hist_source &          hs  = lnav_data.ld_db_source;
 
@@ -1082,7 +1242,6 @@ static string com_summarize(string cmdline, vector<string> &args)
                 {
                     const char *errmsg;
 
-                    fprintf(stderr, "code %d\n", retcode);
                     errmsg = sqlite3_errmsg(lnav_data.ld_db);
                     retval = "error: " + string(errmsg);
                     done   = true;
@@ -1150,6 +1309,83 @@ static string com_add_test(string cmdline, vector<string> &args)
     return retval;
 }
 
+static string com_switch_to_view(string cmdline, vector<string> &args)
+{
+    string retval = "";
+
+    if (args.size() == 0) {
+        args.push_back("viewname");
+    }
+    else if (args.size() > 1) {
+        bool found = false;
+
+        for (int lpc = 0; lnav_view_strings[lpc] && !found; lpc++) {
+            if (strcasecmp(args[1].c_str(), lnav_view_strings[lpc]) == 0) {
+                ensure_view(&lnav_data.ld_views[lpc]);
+                found = true;
+            }
+        }
+
+        if (!found) {
+            retval = "error: invalid view name -- " + args[1];
+        }
+    }
+
+    return retval;
+}
+
+static string com_load_session(string cmdline, vector<string> &args)
+{
+    if (args.empty()) {
+
+    }
+    else {
+        scan_sessions();
+        load_session();
+        lnav_data.ld_views[LNV_LOG].reload_data();
+    }
+
+    return "";
+}
+
+static string com_save_session(string cmdline, vector<string> &args)
+{
+    if (args.empty()) {
+
+    }
+    else {
+        save_session();
+    }
+
+    return "";
+}
+
+static string com_set_min_log_level(string cmdline, vector<string> &args)
+{
+    string retval = "error: expecting log level name";
+
+    if (args.empty()) {
+        args.push_back("levelname");
+    }
+    else if (args.size() == 2) {
+        logline::level_t new_level;
+
+        new_level = logline::string2level(
+            args[1].c_str(), args[1].size(), true);
+        if (lnav_data.ld_level_filter.lf_min_level != new_level) {
+            lnav_data.ld_level_filter.lf_min_level = new_level;
+            lnav_data.ld_log_source.filter_changed();
+
+            rebuild_indexes(true);
+        }
+
+        retval = ("info: minimum log level is now -- " +
+            string(logline::level_names[new_level]));
+    }
+
+    return retval;
+}
+
 void init_lnav_commands(readline_context::command_map_t &cmd_map)
 {
     cmd_map["adjust-log-time"]      = com_adjust_log_time;
@@ -1164,6 +1400,7 @@ void init_lnav_commands(readline_context::command_map_t &cmd_map)
     cmd_map["append-to"]            = com_save_to;
     cmd_map["write-to"]             = com_save_to;
     cmd_map["write-csv-to"]         = com_save_to;
+    cmd_map["write-json-to"]        = com_save_to;
     cmd_map["enable-filter"]        = com_enable_filter;
     cmd_map["disable-filter"]       = com_disable_filter;
     cmd_map["enable-word-wrap"]     = com_enable_word_wrap;
@@ -1173,8 +1410,13 @@ void init_lnav_commands(readline_context::command_map_t &cmd_map)
     cmd_map["open"]                 = com_open;
     cmd_map["close"]                = com_close;
     cmd_map["partition-name"]       = com_partition_name;
+    cmd_map["clear-partition"]      = com_clear_partition;
     cmd_map["session"]              = com_session;
     cmd_map["summarize"]            = com_summarize;
+    cmd_map["switch-to-view"]       = com_switch_to_view;
+    cmd_map["load-session"]         = com_load_session;
+    cmd_map["save-session"]         = com_save_session;
+    cmd_map["set-min-log-level"]    = com_set_min_log_level;
 
     if (getenv("LNAV_SRC") != NULL) {
         cmd_map["add-test"] = com_add_test;
