@@ -40,16 +40,17 @@
 #include <sqlite3.h>
 
 #include "auto_fd.hh"
+#include "ptimec.hh"
 #include "lnav_util.hh"
 
 std::string hash_string(const std::string &str)
 {
-    byte_array<SHA_DIGEST_LENGTH> hash;
-    SHA_CTX context;
+    byte_array<2, uint64> hash;
+    SpookyHash context;
 
-    SHA_Init(&context);
-    SHA_Update(&context, str.c_str(), str.length());
-    SHA_Final(hash.out(), &context);
+    context.Init(0, 0);
+    context.Update(str.c_str(), str.length());
+    context.Final(hash.out(0), hash.out(1));
 
     return hash.to_string();
 }
@@ -196,9 +197,11 @@ time_t tm2sec(const struct tm *t)
         return BAD_DATE;
     }                          /* must have overflowed */
     else {
+#ifdef HAVE_STRUCT_TM_TM_ZONE
         if (t->tm_zone) {
             days -= t->tm_gmtoff;
         }
+#endif
         return days;
     }                          /* must be a valid time */
 }
@@ -237,6 +240,7 @@ const char *std_time_fmt[] = {
     "%a %b %d %H:%M:%S %Z %Y",
     time_fmt_with_zone,
 
+    "%d/%b/%Y:%H:%M:%S +0000",
     "%d/%b/%Y:%H:%M:%S %z",
 
     "%b %d %H:%M:%S",
@@ -260,14 +264,14 @@ const char *date_time_scanner::scan(const char *time_dest,
     const char *retval = NULL;
 
     if (!time_fmt) {
-        time_fmt = std_time_fmt;
+        time_fmt = PTIMEC_FORMAT_STR;
     }
 
     while (next_format(time_fmt,
                        curr_time_fmt,
                        this->dts_fmt_lock)) {
         *tm_out = this->dts_base_tm;
-        if (time_fmt[curr_time_fmt][0] == '+') {
+        if (time_dest[0] == '+') {
             int gmt_int, off;
 
             retval = NULL;
@@ -276,11 +280,14 @@ const char *date_time_scanner::scan(const char *time_dest,
 
                 if (this->dts_local_time) {
                     localtime_r(&gmt, tm_out);
+#ifdef HAVE_STRUCT_TM_TM_ZONE
                     tm_out->tm_zone = NULL;
+#endif
                     tm_out->tm_isdst = 0;
                     gmt = tm2sec(tm_out);
                 }
                 tv_out.tv_sec = gmt;
+                tv_out.tv_usec = 0;
 
                 this->dts_fmt_lock = curr_time_fmt;
                 this->dts_fmt_len = off;
@@ -289,6 +296,38 @@ const char *date_time_scanner::scan(const char *time_dest,
                 break;
             }
         }
+        else if (time_fmt == PTIMEC_FORMAT_STR) {
+            ptime_func func = PTIMEC_FORMATS[curr_time_fmt].pf_func;
+            off_t off = 0;
+
+#ifdef HAVE_STRUCT_TM_TM_ZONE
+            tm_out->tm_zone = NULL;
+#endif
+            if (func(tm_out, time_dest, off, strlen(time_dest))) {
+                retval = &time_dest[off];
+
+                if (tm_out->tm_year < 70) {
+                    tm_out->tm_year = 80;
+                }
+                if (this->dts_local_time) {
+                    time_t gmt = tm2sec(tm_out);
+
+                    localtime_r(&gmt, tm_out);
+#ifdef HAVE_STRUCT_TM_TM_ZONE
+                    tm_out->tm_zone = NULL;
+#endif
+                    tm_out->tm_isdst = 0;
+                }
+                tv_out.tv_sec = tm2sec(tm_out);
+                tv_out.tv_usec = 0;
+
+                this->dts_fmt_lock = curr_time_fmt;
+                this->dts_fmt_len  = retval - time_dest;
+
+                found = true;
+                break;
+            }
+        }
         else if ((retval = strptime(time_dest,
                                     time_fmt[curr_time_fmt],
                                     tm_out)) != NULL) {
@@ -315,9 +354,12 @@ const char *date_time_scanner::scan(const char *time_dest,
                 time_t gmt = tm2sec(tm_out);
 
                 localtime_r(&gmt, tm_out);
+#ifdef HAVE_STRUCT_TM_TM_ZONE
                 tm_out->tm_zone = NULL;
+#endif
                 tm_out->tm_isdst = 0;
             }
+
             tv_out.tv_sec = tm2sec(tm_out);
             tv_out.tv_usec = 0;
 
@@ -355,3 +397,33 @@ const char *date_time_scanner::scan(const char *time_dest,
 
     return retval;
 }
+
+template<typename T>
+size_t strtonum(T &num_out, const char *string, size_t len)
+{
+    size_t retval = 0;
+    T sign = 1;
+
+    num_out = 0;
+    
+    for (; retval < len && isspace(string[retval]); retval++);
+    for (; retval < len && string[retval] == '-'; retval++) {
+        sign *= -1;
+    }
+    for (; retval < len && string[retval] == '+'; retval++);
+    for (; retval < len && isdigit(string[retval]); retval++) {
+        num_out *= 10;
+        num_out += string[retval] - '0';
+    }
+
+    return retval;
+}
+
+template
+size_t strtonum<long long>(long long &num_out, const char *string, size_t len);
+
+template
+size_t strtonum<long>(long &num_out, const char *string, size_t len);
+
+template
+size_t strtonum<int>(int &num_out, const char *string, size_t len);
