@@ -29,6 +29,7 @@
 
 #include "config.h"
 
+#include <math.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -56,7 +57,12 @@ using namespace std;
  *   iostat
  */
 
-const char *logline::level_names[LEVEL__MAX] = {
+string_attr_type logline::L_PREFIX;
+string_attr_type logline::L_TIMESTAMP;
+string_attr_type logline::L_FILE;
+string_attr_type logline::L_PARTITION;
+
+const char *logline::level_names[LEVEL__MAX + 1] = {
     "unknown",
     "trace",
     "debug",
@@ -65,6 +71,8 @@ const char *logline::level_names[LEVEL__MAX] = {
     "error",
     "critical",
     "fatal",
+
+    NULL
 };
 
 static int strncasestr_i(const char *s1, const char *s2, size_t len)
@@ -78,7 +86,7 @@ logline::level_t logline::string2level(const char *levelstr, size_t len, bool ex
 
     int (*cmpfunc)(const char *, const char *, size_t);
 
-    assert(len == (size_t)-1 || (len != (size_t)-1 && exact));
+    require(len == (size_t)-1 || (len != (size_t)-1 && exact));
 
     if (len == (size_t)-1)
         len = strlen(levelstr);
@@ -90,6 +98,11 @@ logline::level_t logline::string2level(const char *levelstr, size_t len, bool ex
         cmpfunc = strncasestr_i;
     }
 
+    if (((len == 1) || ((len > 1) && (levelstr[1] == ' '))) &&
+        (retval = abbrev2level(levelstr, len)) != LEVEL_UNKNOWN) {
+        return retval;
+    }
+
     if (cmpfunc(levelstr, "TRACE", len) == 0) {
         retval = logline::LEVEL_TRACE;
     }
@@ -118,6 +131,49 @@ logline::level_t logline::string2level(const char *levelstr, size_t len, bool ex
     return retval;
 }
 
+logline::level_t logline::abbrev2level(const char *levelstr, size_t len)
+{
+    if (len == -1) {
+        len = strlen(levelstr);
+    }
+
+    if (len == 0) {
+        return LEVEL_UNKNOWN;
+    }
+
+    switch (toupper(levelstr[0])) {
+        case 'T':
+        return LEVEL_TRACE;
+        case 'D':
+        return LEVEL_DEBUG;
+        case 'I':
+        return LEVEL_INFO;
+        case 'W':
+        return LEVEL_WARNING;
+        case 'E':
+        return LEVEL_ERROR;
+        case 'C':
+        return LEVEL_CRITICAL;
+        case 'F':
+        return LEVEL_FATAL;
+    }
+
+    return LEVEL_UNKNOWN;
+}
+
+int logline::levelcmp(const char *l1, size_t l1_len, const char *l2, size_t l2_len)
+{
+    return abbrev2level(l1, l1_len) - abbrev2level(l2, l2_len);
+}
+
+const char *logline_value::value_names[VALUE__MAX] = {
+    "null",
+    "text",
+    "int",
+    "float",
+    "bool"
+};
+
 logline_value::kind_t logline_value::string2kind(const char *kindstr)
 {
     if (strcmp(kindstr, "string") == 0) {
@@ -279,9 +335,17 @@ static int read_json_int(yajlpp_parse_context *ypc, long long val)
         jlu->jlu_format->jlf_line_format;
     string field_name = ypc->get_path_fragment(0);
 
-    if (find_if(line_format.begin(), line_format.end(),
-                json_field_cmp(external_log_format::JLF_VARIABLE,
-                               field_name)) == line_format.end()) {
+    if (field_name == jlu->jlu_format->lf_timestamp_field) {
+        long long divisor = jlu->jlu_format->elf_timestamp_divisor;
+        struct timeval tv;
+
+        tv.tv_sec = val / divisor;
+        tv.tv_usec = (val % divisor) * (1000000.0 / divisor);
+        jlu->jlu_base_line->set_time(tv);
+    }
+    else if (find_if(line_format.begin(), line_format.end(),
+                     json_field_cmp(external_log_format::JLF_VARIABLE,
+                         field_name)) == line_format.end()) {
         jlu->jlu_sub_line_count += 1;
     }
 
@@ -295,9 +359,17 @@ static int read_json_double(yajlpp_parse_context *ypc, double val)
         jlu->jlu_format->jlf_line_format;
     string field_name = ypc->get_path_fragment(0);
 
-    if (find_if(line_format.begin(), line_format.end(),
-                json_field_cmp(external_log_format::JLF_VARIABLE,
-                               field_name)) == line_format.end()) {
+    if (field_name == jlu->jlu_format->lf_timestamp_field) {
+        double divisor = jlu->jlu_format->elf_timestamp_divisor;
+        struct timeval tv;
+
+        tv.tv_sec = val / divisor;
+        tv.tv_usec = fmod(val, divisor) * (1000000.0 / divisor);
+        jlu->jlu_base_line->set_time(tv);
+    }
+    else if (find_if(line_format.begin(), line_format.end(),
+                     json_field_cmp(external_log_format::JLF_VARIABLE,
+                         field_name)) == line_format.end()) {
         jlu->jlu_sub_line_count += 1;
     }
 
@@ -334,15 +406,16 @@ static int json_array_end(void *ctx)
     if (ypc->ypc_path_index_stack.size() == 1) {
         string field_name = ypc->get_path_fragment(0);
         size_t sub_end = yajl_get_bytes_consumed(jlu->jlu_handle);
-        string str = string(&jlu->jlu_line_value[jlu->jlu_sub_start],
-                            sub_end - jlu->jlu_sub_start);
+        tmp_shared_buffer tsb(&jlu->jlu_line_value[jlu->jlu_sub_start],
+            sub_end - jlu->jlu_sub_start);
 
         jlu->jlu_format->jlf_line_values.push_back(
-            logline_value(field_name, str));
+            logline_value(field_name, tsb.tsb_ref));
     }
 
     return 1;
 }
+
 static struct json_path_handler json_log_handlers[] = {
     json_path_handler("^/\\w+$").
     add_cb(read_json_null).
@@ -454,12 +527,21 @@ bool external_log_format::scan(std::vector<logline> &dst,
     int curr_fmt = -1;
 
     while (next_format(this->elf_pattern_order, curr_fmt, this->lf_fmt_lock)) {
-        if (!this->elf_pattern_order[curr_fmt]->p_pcre->match(pc, pi)) {
+        pcrepp *pat = this->elf_pattern_order[curr_fmt]->p_pcre;
+
+        if (!pat->match(pc, pi)) {
             continue;
         }
 
-        pcre_context::capture_t *ts = pc[this->lf_timestamp_field];
-        pcre_context::capture_t *level_cap = pc[this->elf_level_field];
+        if (this->lf_fmt_lock == -1) {
+            this->lf_timestamp_field_index = pat->name_index(
+                this->lf_timestamp_field);
+            this->elf_level_field_index = pat->name_index(
+                this->elf_level_field);
+        }
+
+        pcre_context::capture_t *ts = pc[this->lf_timestamp_field_index];
+        pcre_context::capture_t *level_cap = pc[this->elf_level_field_index];
         const char *ts_str = pi.get_substr_start(ts);
         const char *last;
         struct tm log_time_tm;
@@ -489,19 +571,7 @@ bool external_log_format::scan(std::vector<logline> &dst,
             }
         }
 
-        if (!dst.empty() &&
-            ((dst.back().get_time() - log_tv.tv_sec) > (24 * 60 * 60))) {
-            vector<logline>::iterator iter;
-
-            for (iter = dst.begin(); iter != dst.end(); iter++) {
-                time_t     ot = iter->get_time();
-                struct tm *otm;
-
-                otm           = gmtime(&ot);
-                otm->tm_year -= 1;
-                iter->set_time(tm2sec(otm));
-            }
-        }
+        this->check_for_new_year(dst, log_tv);
 
         dst.push_back(logline(offset, log_tv, level));
 
@@ -513,12 +583,12 @@ bool external_log_format::scan(std::vector<logline> &dst,
     return retval;
 }
 
-void external_log_format::annotate(const std::string &line,
+void external_log_format::annotate(shared_buffer_ref &line,
                                    string_attrs_t &sa,
                                    std::vector<logline_value> &values) const
 {
     pcre_context_static<128> pc;
-    pcre_input pi(line);
+    pcre_input pi(line.get_data(), 0, line.length());
     struct line_range lr;
     pcre_context::capture_t *cap;
 
@@ -534,27 +604,29 @@ void external_log_format::annotate(const std::string &line,
         return;
     }
 
-    cap = pc[this->lf_timestamp_field];
+    cap = pc[this->lf_timestamp_field_index];
     lr.lr_start = cap->c_begin;
     lr.lr_end = cap->c_end;
-    sa[lr].insert(make_string_attr("timestamp", 0));
+    sa.push_back(string_attr(lr, &logline::L_TIMESTAMP));
 
-    cap = pc[this->elf_body_field];
+    cap = pc[this->elf_body_field_index];
     if (cap != NULL && cap->c_begin != -1) {
         lr.lr_start = cap->c_begin;
         lr.lr_end = cap->c_end;
     }
     else {
-        lr.lr_start = line.size();
-        lr.lr_end = line.size();
+        lr.lr_start = line.length();
+        lr.lr_end = line.length();
     }
-    sa[lr].insert(make_string_attr("body", 0));
+    sa.push_back(string_attr(lr, &textview_curses::SA_BODY));
 
     view_colors &vc = view_colors::singleton();
 
     for (size_t lpc = 0; lpc < pat.p_value_by_index.size(); lpc++) {
         const value_def &vd = pat.p_value_by_index[lpc];
         const struct scaling_factor *scaling = NULL;
+        pcre_context::capture_t *cap = pc[vd.vd_index];
+        shared_buffer_ref field;
 
         if (vd.vd_unit_field_index >= 0) {
             pcre_context::iterator unit_cap = pc[vd.vd_unit_field_index];
@@ -572,19 +644,22 @@ void external_log_format::annotate(const std::string &line,
             }
         }
 
+        field.subset(line, cap->c_begin, cap->length());
+
         values.push_back(logline_value(vd.vd_name,
                          vd.vd_kind,
-                         pi.get_substr(pc[vd.vd_index]),
+                         field,
                          vd.vd_identifier,
                          scaling,
                          vd.vd_column,
-                         pc[vd.vd_index]->c_begin,
-                         pc[vd.vd_index]->c_end));
+                         cap->c_begin,
+                         cap->c_end));
 
         if (pc[vd.vd_index]->c_begin != -1 && vd.vd_identifier) {
             lr.lr_start = pc[vd.vd_index]->c_begin;
             lr.lr_end = pc[vd.vd_index]->c_end;
-            sa[lr].insert(make_string_attr("style", vc.attrs_for_ident(pi.get_substr_start(pc[vd.vd_index]), lr.length())));
+            sa.push_back(string_attr(lr, &view_curses::VC_STYLE,
+                vc.attrs_for_ident(pi.get_substr_start(pc[vd.vd_index]), lr.length())));
         }
     }
 }
@@ -625,28 +700,30 @@ static int rewrite_json_field(yajlpp_parse_context *ypc, const unsigned char *st
 {
     json_log_userdata *jlu = (json_log_userdata *)ypc->ypc_userdata;
     string field_name = ypc->get_path_fragment(0);
-    string val = string((const char *)str, len);
 
     if (field_name == jlu->jlu_format->lf_timestamp_field) {
         char time_buf[64];
 
-        sql_strftime(time_buf, sizeof(time_buf), jlu->jlu_line->get_timeval());
-        val = time_buf;
+        sql_strftime(time_buf, sizeof(time_buf),
+            jlu->jlu_line->get_timeval(), 'T');
+        tmp_shared_buffer tsb(time_buf);
+        jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, tsb.tsb_ref));
     }
-    if (field_name == jlu->jlu_format->elf_body_field) {
-        jlu->jlu_format->jlf_line_values.push_back(logline_value("body", val));
+    else {
+        tmp_shared_buffer tsb((const char *)str, len);
+
+        if (field_name == jlu->jlu_format->elf_body_field) {
+            jlu->jlu_format->jlf_line_values.push_back(logline_value("body", tsb.tsb_ref));
+        }
+        jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, tsb.tsb_ref));
     }
-    jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, val));
 
     return 1;
 }
 
-void external_log_format::get_subline(const logline &ll,
-                                      const char *line, size_t len,
-                                      ostringstream &stream_out)
+void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr)
 {
     if (!this->jlf_json) {
-        stream_out.write(line, len);
         return;
     }
 
@@ -656,6 +733,7 @@ void external_log_format::get_subline(const logline &ll,
         view_colors &vc = view_colors::singleton();
         json_log_userdata jlu;
 
+        this->jlf_share_manager.invalidate_refs();
         this->jlf_cached_line.clear();
         this->jlf_line_values.clear();
         this->jlf_line_offsets.clear();
@@ -674,9 +752,10 @@ void external_log_format::get_subline(const logline &ll,
         jlu.jlu_format = this;
         jlu.jlu_line = &ll;
         jlu.jlu_handle = handle;
-        jlu.jlu_line_value = line;
+        jlu.jlu_line_value = sbr.get_data();
 
-        yajl_status parse_status = yajl_parse(handle.in(), (const unsigned char *)line, len);
+        yajl_status parse_status = yajl_parse(handle.in(),
+            (const unsigned char *)sbr.get_data(), sbr.length());
         if (parse_status == yajl_status_ok &&
             yajl_complete_parse(handle.in()) == yajl_status_ok) {
             std::vector<logline_value>::iterator lv_iter;
@@ -720,17 +799,34 @@ void external_log_format::get_subline(const logline &ll,
                             lr.lr_end = lines.tellp();
                         else
                             lr.lr_end = lr.lr_start + nl_pos;
-                        if (lv_iter->lv_name == this->lf_timestamp_field)
-                            this->jlf_line_attrs[lr].insert(make_string_attr("timestamp", 0));
-                        else if (lv_iter->lv_name == this->elf_body_field)
-                            this->jlf_line_attrs[lr].insert(make_string_attr("body", 0));
+                        if (lv_iter->lv_name == this->lf_timestamp_field) {
+                            this->jlf_line_attrs.push_back(
+                                string_attr(lr, &logline::L_TIMESTAMP));
+                        }
+                        else if (lv_iter->lv_name == this->elf_body_field) {
+                            this->jlf_line_attrs.push_back(
+                                string_attr(lr, &textview_curses::SA_BODY));
+                        }
                         else if (lv_iter->lv_identifier) {
-                            this->jlf_line_attrs[lr].insert(make_string_attr("style", vc.attrs_for_ident(str.c_str(), lr.length())));
+                            this->jlf_line_attrs.push_back(
+                                string_attr(lr, &view_curses::VC_STYLE,
+                                    vc.attrs_for_ident(str.c_str(), lr.length())));
                         }
                         lv_iter->lv_origin = lr;
                         used_values[distance(this->jlf_line_values.begin(),
                                              lv_iter)] = true;
                     }
+                    else if (iter->jfe_value == "__timestamp__") {
+                        struct line_range lr;
+                        char ts[64];
+
+                        sql_strftime(ts, sizeof(ts), ll.get_timeval(), 'T');
+                        lr.lr_start = lines.tellp();
+                        lines << ts;
+                        lr.lr_end = lines.tellp();
+                        this->jlf_line_attrs.push_back(
+                            string_attr(lr, &logline::L_TIMESTAMP));
+                    }
                     else {
                         lines << iter->jfe_default_value;
                     }
@@ -742,6 +838,7 @@ void external_log_format::get_subline(const logline &ll,
                 logline_value &lv = this->jlf_line_values[lpc];
 
                 if (used_values[lpc] ||
+                    lv.lv_name == this->lf_timestamp_field ||
                     lv.lv_name == "body" ||
                     lv.lv_name == this->elf_level_field) {
                     continue;
@@ -788,8 +885,9 @@ void external_log_format::get_subline(const logline &ll,
         this_off += 1;
     next_off = this->jlf_line_offsets[ll.get_sub_offset() + 1];
 
-    stream_out.write(this->jlf_cached_line.c_str() + this_off,
-                     next_off - this_off);
+    sbr.share(this->jlf_share_manager,
+              (char *)this->jlf_cached_line.c_str() + this_off,
+              next_off - this_off);
 }
 
 void external_log_format::build(std::vector<std::string> &errors)
@@ -905,7 +1003,7 @@ void external_log_format::build(std::vector<std::string> &errors)
         bool found = false;
 
         for (std::vector<pattern *>::iterator pat_iter = this->elf_pattern_order.begin();
-             pat_iter != this->elf_pattern_order.end();
+             pat_iter != this->elf_pattern_order.end() && !found;
              ++pat_iter) {
             pattern &pat = *(*pat_iter);
 
@@ -913,8 +1011,16 @@ void external_log_format::build(std::vector<std::string> &errors)
                 continue;
 
             if (pat.p_pcre->match(pc, pi)) {
-                found = true;
-                break;
+                const char *ts = pi.get_substr_start(
+                    pc[this->lf_timestamp_field]);
+                date_time_scanner dts;
+                struct timeval tv;
+                struct tm tm;
+
+                if (dts.scan(ts, NULL, &tm, tv) != NULL) {
+                    found = true;
+                    break;
+                }
             }
         }
 
@@ -987,7 +1093,8 @@ public:
                 type = SQLITE_INTEGER;
                 break;
             case logline_value::VALUE_UNKNOWN:
-                assert(0);
+            case logline_value::VALUE__MAX:
+                ensure(0);
                 break;
             }
             cols[vd.vd_column].vc_name = vd.vd_name.c_str();
