@@ -45,6 +45,10 @@
 #include "format-text-files.hh"
 #include "default-log-formats-json.hh"
 
+#ifndef SYSCONFDIR
+#define SYSCONFDIR "/usr/etc"
+#endif
+
 using namespace std;
 
 static map<string, external_log_format *> LOG_FORMATS;
@@ -87,6 +91,42 @@ static int read_format_bool(yajlpp_parse_context *ypc, int val)
     return 1;
 }
 
+static int read_format_double(yajlpp_parse_context *ypc, double val)
+{
+    external_log_format *elf = ensure_format(ypc);
+    string field_name = ypc->get_path_fragment(1);
+
+    if (field_name == "timestamp-divisor") {
+        if (val <= 0) {
+            fprintf(stderr, "error:%s: timestamp-divisor cannot be less "
+                "than or equal to zero\n",
+                ypc->get_path_fragment(0).c_str());
+            return 0;
+        }
+        elf->elf_timestamp_divisor = val;
+    }
+
+    return 1;
+}
+
+static int read_format_int(yajlpp_parse_context *ypc, long long val)
+{
+    external_log_format *elf = ensure_format(ypc);
+    string field_name = ypc->get_path_fragment(1);
+
+    if (field_name == "timestamp-divisor") {
+        if (val <= 0) {
+            fprintf(stderr, "error:%s: timestamp-divisor cannot be less "
+                "than or equal to zero\n",
+                ypc->get_path_fragment(0).c_str());
+            return 0;
+        }
+        elf->elf_timestamp_divisor = val;
+    }
+
+    return 1;
+}
+
 static int read_format_field(yajlpp_parse_context *ypc, const unsigned char *str, size_t len)
 {
     external_log_format *elf = ensure_format(ypc);
@@ -322,6 +362,8 @@ static int read_json_variable_num(yajlpp_parse_context *ypc, long long val)
 static struct json_path_handler format_handlers[] = {
     json_path_handler("^/\\w+/regex/[^/]+/pattern$", read_format_regex),
     json_path_handler("^/\\w+/(json|convert-to-local-time)$", read_format_bool),
+    json_path_handler("^/\\w+/timestamp-divisor$", read_format_double)
+        .add_cb(read_format_int),
     json_path_handler("^/\\w+/(file-pattern|level-field|timestamp-field|body-field|url|title|description)$",
                       read_format_field),
     json_path_handler("^/\\w+/level/"
@@ -343,59 +385,46 @@ static struct json_path_handler format_handlers[] = {
     json_path_handler()
 };
 
-void load_formats(std::vector<std::string> &errors)
+static void write_sample_file(void)
 {
-    string default_source = string(dotlnav_path("formats") + "/default/");
-    yajlpp_parse_context ypc_builtin(default_source, format_handlers);
-    std::vector<std::string> retval;
+    string sample_path = dotlnav_path("formats/default/default-formats.json.sample");
+    auto_fd sample_fd;
+
+    if ((sample_fd = open(sample_path.c_str(),
+                          O_WRONLY|O_TRUNC|O_CREAT,
+                          0644)) == -1) {
+        perror("error: unable to write default format file");
+    }
+    else {
+        write(sample_fd.get(),
+              default_log_formats_json,
+              strlen(default_log_formats_json));
+    }
+
+    string sh_path = dotlnav_path("formats/default/dump-pid.sh");
+    auto_fd sh_fd;
+
+    if ((sh_fd = open(sh_path.c_str(), O_WRONLY|O_TRUNC|O_CREAT, 0755)) == -1) {
+        perror("error: unable to write default text file");
+    }
+    else {
+        write(sh_fd.get(), dump_pid_sh, strlen(dump_pid_sh));
+    }
+}
+
+static void load_from_path(const string &path, std::vector<string> &errors)
+{
+    string format_path = path + "/formats/*/*.json";
+    static_root_mem<glob_t, globfree> gl;
     yajl_handle handle;
 
-    {
-        string sample_path = dotlnav_path("formats/default/default-formats.json.sample");
-        auto_fd sample_fd;
-
-        if ((sample_fd = open(sample_path.c_str(),
-                              O_WRONLY|O_TRUNC|O_CREAT,
-                              0644)) == -1) {
-            perror("error: unable to write default format file");
-        }
-        else {
-            write(sample_fd.get(),
-                  default_log_formats_json,
-                  strlen(default_log_formats_json));
-        }
-    }
-
-    {
-        string sh_path = dotlnav_path("formats/default/dump-pid.sh");
-        auto_fd sh_fd;
-
-        if ((sh_fd = open(sh_path.c_str(), O_WRONLY|O_TRUNC|O_CREAT, 0755)) == -1) {
-            perror("error: unable to write default text file");
-        }
-        else {
-            write(sh_fd.get(), dump_pid_sh, strlen(dump_pid_sh));
-        }
-    }
-
-    handle = yajl_alloc(&ypc_builtin.ypc_callbacks, NULL, &ypc_builtin);
-    if (yajl_parse(handle,
-                   (const unsigned char *)default_log_formats_json,
-                   strlen(default_log_formats_json)) != yajl_status_ok) {
-        errors.push_back("builtin: invalid json -- " +
-            string((char *)yajl_get_error(handle, 1, (unsigned char *)default_log_formats_json, strlen(default_log_formats_json))));
-    }
-    yajl_complete_parse(handle);
-    yajl_free(handle);
-
-    string format_path = dotlnav_path("formats/*/*.json");
-    static_root_mem<glob_t, globfree> gl;
-
+    log_info("loading formats from path: %s", format_path.c_str());
     if (glob(format_path.c_str(), 0, NULL, gl.inout()) == 0) {
         for (int lpc = 0; lpc < (int)gl->gl_pathc; lpc++) {
             string filename(gl->gl_pathv[lpc]);
             auto_fd fd;
 
+            log_info("loading formats from file: %s", filename.c_str());
             yajlpp_parse_context ypc(filename, format_handlers);
             if ((fd = open(gl->gl_pathv[lpc], O_RDONLY)) == -1) {
                 char errmsg[1024];
@@ -439,6 +468,37 @@ void load_formats(std::vector<std::string> &errors)
             }
         }
     }
+}
+
+void load_formats(const std::vector<std::string> &extra_paths,
+                  std::vector<std::string> &errors)
+{
+    string default_source = string(dotlnav_path("formats") + "/default/");
+    yajlpp_parse_context ypc_builtin(default_source, format_handlers);
+    std::vector<std::string> retval;
+    yajl_handle handle;
+
+    write_sample_file();
+
+    handle = yajl_alloc(&ypc_builtin.ypc_callbacks, NULL, &ypc_builtin);
+    if (yajl_parse(handle,
+                   (const unsigned char *)default_log_formats_json,
+                   strlen(default_log_formats_json)) != yajl_status_ok) {
+        errors.push_back("builtin: invalid json -- " +
+            string((char *)yajl_get_error(handle, 1, (unsigned char *)default_log_formats_json, strlen(default_log_formats_json))));
+    }
+    yajl_complete_parse(handle);
+    yajl_free(handle);
+
+    load_from_path("/etc/lnav", errors);
+    load_from_path(SYSCONFDIR "/lnav", errors);
+    load_from_path(dotlnav_path(""), errors);
+
+    for (vector<string>::const_iterator path_iter = extra_paths.begin();
+         path_iter != extra_paths.end();
+         ++path_iter) {
+        load_from_path(*path_iter, errors);
+    }
 
     for (map<string, external_log_format *>::iterator iter = LOG_FORMATS.begin();
          iter != LOG_FORMATS.end();
