@@ -33,9 +33,9 @@
 
 #include "k_merge_tree.h"
 #include "lnav_util.hh"
+#include "log_accel.hh"
 #include "logfile_sub_source.hh"
 
-
 using namespace std;
 
 #if 0
@@ -149,9 +149,9 @@ logfile *logfile_sub_source::find(const char *fn,
     return retval;
 }
 
-vis_line_t logfile_sub_source::find_from_time(time_t start)
+vis_line_t logfile_sub_source::find_from_time(const struct timeval &start)
 {
-    vector<content_line_t>::iterator lb;
+    vector<indexed_content>::iterator lb;
     vis_line_t retval(-1);
 
     lb = lower_bound(this->lss_index.begin(),
@@ -172,10 +172,10 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
 {
     content_line_t line(0);
 
-    assert(row >= 0);
-    assert((size_t)row < this->lss_index.size());
+    require(row >= 0);
+    require((size_t)row < this->lss_index.size());
 
-    line = this->lss_index[row];
+    line = this->at(vis_line_t(row));
     this->lss_token_file   = this->find(line);
     this->lss_token_line   = this->lss_token_file->begin() + line;
     this->lss_token_offset = 0;
@@ -186,6 +186,7 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
         return;
     }
 
+    this->lss_share_manager.invalidate_refs();
     this->lss_token_value =
         this->lss_token_file->read_line(this->lss_token_line);
 
@@ -202,6 +203,7 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
         log_format *format = this->lss_token_file->get_format();
 
         if (format->lf_date_time.dts_fmt_lock != -1) {
+            shared_buffer_ref sbr;
             std::vector<logline_value> line_values;
             struct timeval adjusted_time;
             string_attrs_t sa;
@@ -214,11 +216,13 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
                      fmt,
                      gmtime(&adjusted_time.tv_sec));
 
-            format->annotate(this->lss_token_value, sa, line_values);
+            sbr.share(this->lss_share_manager,
+                (char *)this->lss_token_value.c_str(), this->lss_token_value.size());
+            format->annotate(sbr, sa, line_values);
 
             struct line_range time_range;
 
-            time_range = find_string_attr_range(sa, "timestamp");
+            time_range = find_string_attr_range(sa, &logline::L_TIMESTAMP);
             if (time_range.lr_start != -1) {
                 const char *last = value_out.c_str();
                 int len = strlen(buffer);
@@ -238,7 +242,7 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
     }
 
     if (this->lss_flags & F_TIME_OFFSET) {
-        long long start_millis, curr_millis;
+        int64_t start_millis, curr_millis;
 
         vis_line_t prev_mark =
             tc.get_bookmarks()[&textview_curses::BM_USER].prev(vis_line_t(row));
@@ -247,11 +251,9 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
         }
 
         logline *first_line = this->find_line(this->at(prev_mark));
-        start_millis = first_line->get_time() * 1000 +
-                       first_line->get_millis();
-        curr_millis = this->lss_token_line->get_time() * 1000 +
-                      this->lss_token_line->get_millis();
-        long long diff = curr_millis - start_millis;
+        start_millis = first_line->get_time_in_millis();
+        curr_millis = this->lss_token_line->get_time_in_millis();
+        int64_t diff = curr_millis - start_millis;
 
         /* 24h22m33s111 */
 
@@ -327,7 +329,7 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
     }
 
     if ((row + 1) < (int)this->lss_index.size()) {
-        next_line = this->find_line(this->lss_index[row + 1]);
+        next_line = this->find_line(this->at(vis_line_t(row + 1)));
     }
 
     if (next_line != NULL &&
@@ -340,13 +342,17 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
     std::vector<logline_value> line_values;
     
     if (!(this->lss_token_line->get_level() & logline::LEVEL_CONTINUED)) {
-        format->annotate(this->lss_token_value, value_out, line_values);
+        shared_buffer_ref sbr;
+
+        sbr.share(this->lss_share_manager,
+            (char *)this->lss_token_value.c_str(), this->lss_token_value.size());
+        format->annotate(sbr, value_out, line_values);
     }
 
     lr.lr_start = time_offset_end + this->lss_token_date_end;
     lr.lr_end   = -1;
 
-    value_out[lr].insert(make_string_attr("style", attrs));
+    value_out.push_back(string_attr(lr, &view_curses::VC_STYLE, attrs));
 
     if (this->lss_flags & F_TIME_OFFSET) {
         time_offset_end = 13;
@@ -356,23 +362,42 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
         for (string_attrs_t::iterator iter = value_out.begin();
              iter != value_out.end();
              ++iter) {
-            struct line_range *existing_lr = (line_range *)&iter->first;
+            struct line_range *existing_lr = (line_range *)&iter->sa_range;
 
             existing_lr->lr_start += time_offset_end;
-            if (existing_lr->lr_end != -1)
+            if (existing_lr->lr_end != -1) {
                 existing_lr->lr_end += time_offset_end;
+            }
         }
 
-        attrs = vc.attrs_for_role(view_colors::VCR_OK);
-        value_out[lr].insert(make_string_attr("style", attrs));
+        // attrs = vc.attrs_for_role(view_colors::VCR_OK);
+        attrs = view_colors::ansi_color_pair(COLOR_CYAN, COLOR_BLACK);
+        value_out.push_back(string_attr(lr, &view_curses::VC_STYLE, attrs));
+        value_out.push_back(string_attr(line_range(12, 13),
+            &view_curses::VC_GRAPHIC, ACS_VLINE));
+
+        int bar_attrs = 0;
+
+        switch (this->get_line_accel_direction(vis_line_t(row))) {
+        case log_accel::A_STEADY:
+            break;
+        case log_accel::A_DECEL:
+            bar_attrs = view_colors::ansi_color_pair(COLOR_RED, COLOR_BLACK);
+            break;
+        case log_accel::A_ACCEL:
+            bar_attrs = view_colors::ansi_color_pair(COLOR_GREEN, COLOR_BLACK);
+            break;
+        }
+        value_out.push_back(
+            string_attr(line_range(12, 13), &view_curses::VC_STYLE, bar_attrs));
     }
 
     lr.lr_start = 0;
     lr.lr_end   = -1;
-    value_out[lr].insert(make_string_attr("file", this->lss_token_file));
+    value_out.push_back(string_attr(lr, &logline::L_FILE, this->lss_token_file));
 
     {
-        bookmark_vector<vis_line_t> &bv = lv.get_bookmarks()[&textview_curses::BM_USER];
+        bookmark_vector<vis_line_t> &bv = lv.get_bookmarks()[&textview_curses::BM_PARTITION];
         bookmark_vector<vis_line_t>::iterator bv_iter;
 
         bv_iter = lower_bound(bv.begin(), bv.end(), vis_line_t(row + 1));
@@ -385,30 +410,28 @@ void logfile_sub_source::text_attrs_for_line(textview_curses &lv,
                 != this->lss_user_mark_metadata.end()) {
                 lr.lr_start = 0;
                 lr.lr_end   = -1;
-                value_out[lr].insert(make_string_attr("partition", &bm_iter->second));
+                value_out.push_back(string_attr(lr, &logline::L_PARTITION, &bm_iter->second));
             }
         }
     }
 
     if (this->lss_token_file->is_time_adjusted()) {
-        struct line_range time_range = find_string_attr_range(value_out,
-                                                              "timestamp");
+        struct line_range time_range = find_string_attr_range(
+            value_out, &logline::L_TIMESTAMP);
 
         if (time_range.lr_end != -1) {
             attrs = vc.attrs_for_role(view_colors::VCR_ADJUSTED_TIME);
-            value_out[time_range].insert(make_string_attr("style", attrs));
+            value_out.push_back(string_attr(time_range, &view_curses::VC_STYLE, attrs));
         }
     }
     else if ((((this->lss_token_line->get_time() / (5 * 60)) % 2) == 0) &&
              !(this->lss_token_line->get_level() & logline::LEVEL_CONTINUED)) {
-        struct line_range time_range = find_string_attr_range(value_out,
-                                                              "timestamp");
+        struct line_range time_range = find_string_attr_range(
+            value_out, &logline::L_TIMESTAMP);
 
         if (time_range.lr_end != -1) {
-            time_range.lr_start += time_offset_end;
-            time_range.lr_end   += time_offset_end;
             attrs = vc.attrs_for_role(view_colors::VCR_ALT_ROW);
-            value_out[time_range].insert(make_string_attr("style", attrs));
+            value_out.push_back(string_attr(time_range, &view_curses::VC_STYLE, attrs));
         }
     }
 }
@@ -488,17 +511,21 @@ bool logfile_sub_source::rebuild_index(observer *obs, bool force)
                                     line_index);
 
             if (!(lf_iter->get_level() & logline::LEVEL_CONTINUED)) {
-                if (action_for_prev_line == logfile_filter::INCLUDE) {
-                    while (last_owner->ld_indexing.ld_start <=
+                if (last_owner != NULL) {
+                    if (action_for_prev_line != logfile_filter::EXCLUDE) {
+                        // Append all of the lines from the previous message.
+                        while (last_owner->ld_indexing.ld_start <=
                            last_owner->ld_indexing.ld_last) {
-                        this->lss_index.push_back(last_owner->ld_indexing.ld_start);
-                        ++last_owner->ld_indexing.ld_start;
+                            this->lss_index.push_back(last_owner->ld_indexing.ld_start);
+                            ++last_owner->ld_indexing.ld_start;
+                        }
+                    }
+                    else {
+                        this->lss_filtered_count += 1;
                     }
                 }
-                else if (action_for_prev_line == logfile_filter::EXCLUDE) {
-                    this->lss_filtered_count += 1;
-                }
                 ld->ld_indexing.ld_start = con_line;
+                action_for_prev_line = logfile_filter::MAYBE;
             }
 
             if (obs != NULL) {
@@ -509,8 +536,18 @@ bool logfile_sub_source::rebuild_index(observer *obs, bool force)
             }
 
             ld->ld_indexing.ld_last = con_line;
-            action_for_prev_line = ld->ld_file->check_filter(
-                lf_iter, this->lss_filter_generation, this->lss_filters);
+            if (action_for_prev_line != logfile_filter::INCLUDE) {
+                // We haven't decided to include yet, so check the filter again.
+                // Once we decide to include a sub-line, the whole log message will
+                // always be included.
+                logfile_filter::type_t action_for_this_line = ld->ld_file->check_filter(
+                    lf_iter, this->lss_filter_generation, this->lss_filters);
+
+                // Only record the filter results for this line if it's not decisive.
+                if (action_for_this_line != logfile_filter::MAYBE) {
+                    action_for_prev_line = action_for_this_line;
+                }
+            }
 
             last_owner = ld;
 
@@ -522,15 +559,17 @@ bool logfile_sub_source::rebuild_index(observer *obs, bool force)
                                               total_lines);
         }
 
-        if (action_for_prev_line == logfile_filter::INCLUDE) {
-            while (last_owner->ld_indexing.ld_start <=
+        if (last_owner != NULL) {
+            if (action_for_prev_line != logfile_filter::EXCLUDE) {
+                while (last_owner->ld_indexing.ld_start <=
                    last_owner->ld_indexing.ld_last) {
-                this->lss_index.push_back(last_owner->ld_indexing.ld_start);
-                ++last_owner->ld_indexing.ld_start;
+                    this->lss_index.push_back(last_owner->ld_indexing.ld_start);
+                    ++last_owner->ld_indexing.ld_start;
+                }
+            }
+            else {
+                this->lss_filtered_count += 1;
             }
-        }
-        else if (action_for_prev_line == logfile_filter::EXCLUDE) {
-            this->lss_filtered_count += 1;
         }
 
         for (iter = this->lss_files.begin();
@@ -563,25 +602,34 @@ void logfile_sub_source::text_update_marks(vis_bookmarks &bm)
     }
 
     for (; vl < (int)this->lss_index.size(); ++vl) {
-        content_line_t cl = this->lss_index[vl];
+        const content_line_t orig_cl = this->at(vl);
+        content_line_t cl = orig_cl;
         logfile *      lf;
 
+        lf = this->find(cl);
+
         for (bookmarks<content_line_t>::type::iterator iter =
                  this->lss_user_marks.begin();
              iter != this->lss_user_marks.end();
              ++iter) {
-            if (binary_search(iter->second.begin(), iter->second.end(), cl)) {
+            if (binary_search(iter->second.begin(),
+                              iter->second.end(),
+                              orig_cl)) {
                 bm[iter->first].insert_once(vl);
+
+                if (iter->first == &textview_curses::BM_USER) {
+                    logfile::iterator ll = lf->begin() + cl;
+
+                    ll->set_mark(true);
+                }
             }
         }
 
-        lf = this->find(cl);
-
         if (lf != last_file) {
             bm[&BM_FILES].insert_once(vl);
         }
 
-        switch ((*lf)[cl].get_level() & ~logline::LEVEL_MULTILINE) {
+        switch ((*lf)[cl].get_level() & ~logline::LEVEL_MARK) {
         case logline::LEVEL_WARNING:
             bm[&BM_WARNINGS].insert_once(vl);
             break;
@@ -600,6 +648,29 @@ void logfile_sub_source::text_update_marks(vis_bookmarks &bm)
     }
 }
 
+log_accel::direction_t logfile_sub_source::get_line_accel_direction(
+    vis_line_t vl)
+{
+    log_accel la;
+
+    while (vl >= 0) {
+        logline *curr_line = this->find_line(this->at(vl));
+
+        if (curr_line->is_continued()) {
+            --vl;
+            continue;
+        }
+
+        if (!la.add_point(curr_line->get_time_in_millis())) {
+            break;
+        }
+
+        --vl;
+    }
+
+    return la.get_direction();
+}
+
 #if 0
 void logfile_sub_source::handle_scroll(listview_curses *lc)
 {
