@@ -31,7 +31,6 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -45,6 +44,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 
+#include "lnav_log.hh"
 #include "piper_proc.hh"
 #include "line_buffer.hh"
 
@@ -69,7 +69,7 @@ static int write_timestamp(int fd, off_t woff)
 piper_proc::piper_proc(int pipefd, bool timestamp, const char *filename)
     : pp_fd(-1), pp_child(-1)
 {
-    assert(pipefd >= 0);
+    require(pipefd >= 0);
 
     if (filename) {
         if ((this->pp_fd =
@@ -106,10 +106,8 @@ piper_proc::piper_proc(int pipefd, bool timestamp, const char *filename)
     {
         auto_fd     infd(pipefd);
         line_buffer lb;
-        off_t       woff = 0;
-        off_t       off  = 0;
-        char *      line;
-        size_t      len;
+        off_t       woff = 0, last_woff = 0;
+        off_t       off  = 0, last_off = 0;
         int nullfd;
 
         nullfd = open("/dev/null", O_RDWR);
@@ -117,14 +115,17 @@ piper_proc::piper_proc(int pipefd, bool timestamp, const char *filename)
         fcntl(infd.get(), F_SETFL, O_NONBLOCK);
         lb.set_fd(infd);
         do {
+            line_value lv;
             fd_set rready;
 
             FD_ZERO(&rready);
             FD_SET(lb.get_fd(), &rready);
             select(lb.get_fd() + 1, &rready, NULL, NULL, NULL);
-            while ((line = lb.read_line(off, len)) != NULL) {
+            last_off = off;
+            while (lb.read_line(off, lv, true)) {
                 int wrc;
 
+                last_woff = woff;
                 if (timestamp) {
                     wrc = write_timestamp(this->pp_fd, woff);
                     if (wrc == -1) {
@@ -134,19 +135,23 @@ piper_proc::piper_proc(int pipefd, bool timestamp, const char *filename)
                     woff += wrc;
                 }
 
-                line[len] = '\n';
-
                 /* Need to do pwrite here since the fd is used by the main
                  * lnav process as well.
                  */
-                wrc = pwrite(this->pp_fd, line, len + 1, woff);
+                wrc = pwrite(this->pp_fd, lv.lv_start, lv.lv_len, woff);
                 if (wrc == -1) {
                     perror("Unable to write to output file for stdin");
                     break;
                 }
                 woff += wrc;
+
+                if (lv.lv_start[lv.lv_len - 1] != '\n') {
+                    off = last_off;
+                    woff = last_woff;
+                }
+                last_off = off;
             }
-        } while (lb.get_file_size() == (size_t)-1);
+        } while (lb.get_file_size() == (ssize_t)-1);
 
         if (timestamp) {
             int wrc;
@@ -164,10 +169,9 @@ piper_proc::piper_proc(int pipefd, bool timestamp, const char *filename)
                 perror("Unable to write to output file for stdin");
                 break;
             }
-            woff += wrc;
         }
     }
-        exit(0);
+        _exit(0);
         break;
 
     default:
