@@ -55,7 +55,9 @@
 
 #include <string>
 
+#include "pcrepp.hh"
 #include "auto_mem.hh"
+#include "lnav_log.hh"
 #include "ansi_scrubber.hh"
 #include "readline_curses.hh"
 
@@ -68,8 +70,20 @@ static readline_curses *child_this;
 static sig_atomic_t     looping      = 1;
 static const int        HISTORY_SIZE = 256;
 
+static const char *RL_INIT[] = {
+    /*
+     * XXX Need to keep the input on a single line since the display screws
+     * up if it wraps around.
+     */
+    "set horizontal-scroll-mode on",
+
+    NULL
+};
+
 readline_context *readline_context::loaded_context;
 set<string> *     readline_context::arg_possibilities;
+static string last_match_str;
+static bool last_match_str_valid;
 
 static void sigalrm(int sig)
 {
@@ -93,26 +107,86 @@ static void line_ready_tramp(char *line)
     rl_callback_handler_remove();
 }
 
-static int reliable_send(int sock, char *buf, size_t len)
+static int sendall(int sock, const char *buf, size_t len)
 {
-    int retval;
+    off_t offset = 0;
 
-    while ((retval = send(sock, buf, len, 0)) == -1) {
-        if (errno == ENOBUFS) {
-            fd_set ready_wfds;
+    while (len > 0) {
+        int rc = send(sock, &buf[offset], len, 0);
 
-            FD_ZERO(&ready_wfds);
-            FD_SET(sock, &ready_wfds);
-            select(sock + 1, NULL, &ready_wfds, NULL, NULL);
-        }
-        else if (errno == EINTR) {
-            continue;
+        if (rc == -1) {
+            switch (errno) {
+            case EAGAIN:
+            case EINTR:
+                break;
+            default:
+                return -1;
+            }
         }
         else {
-            break;
+            len -= rc;
+            offset += rc;
         }
     }
 
+    return 0;
+}
+
+static int sendstring(int sock, const char *buf, size_t len)
+{
+    if (sendall(sock, (char *)&len, sizeof(len)) == -1) {
+        return -1;
+    }
+    else if (sendall(sock, buf, len) == -1) {
+        return -1;
+    }
+
+    return 0;
+}
+
+static int recvall(int sock, char *buf, size_t len)
+{
+    off_t offset = 0;
+
+    while (len > 0) {
+        int rc = recv(sock, &buf[offset], len, 0);
+
+        if (rc == -1) {
+            switch (errno) {
+            case EAGAIN:
+            case EINTR:
+                break;
+            default:
+                return -1;
+            }
+        }
+        else if (rc == 0) {
+            errno = EIO;
+            return -1;
+        }
+        else {
+            len -= rc;
+            offset += rc;
+        }
+    }
+
+    return 0;
+}
+
+static ssize_t recvstring(int sock, char *buf, size_t len)
+{
+    ssize_t retval;
+
+    if (recvall(sock, (char *)&retval, sizeof(retval)) == -1) {
+        return -1;
+    }
+    else if (retval > len) {
+        return -1;
+    }
+    else if (recvall(sock, buf, retval) == -1) {
+        return -1;
+    }
+
     return retval;
 }
 
@@ -134,23 +208,31 @@ char *readline_context::completion_generator(const char *text, int state)
                  ++iter) {
                 int (*cmpfunc)(const char *, const char *, size_t);
 
-                fprintf(stderr, " cmp %s %s\n", text, iter->c_str());
                 cmpfunc = (loaded_context->is_case_sensitive() ?
                            strncmp : strncasecmp);
                 if (cmpfunc(text, iter->c_str(), len) == 0) {
-                    fprintf(stderr, "match!!! %d %s  %s\n", len, text,
-                            iter->c_str());
                     matches.push_back(*iter);
                 }
             }
         }
+
+        if (matches.size() == 1) {
+            if (strcmp(text, matches[0].c_str()) == 0) {
+                matches.pop_back();
+            }
+
+            last_match_str_valid = false;
+            if (sendstring(child_this->rc_command_pipe[readline_curses::RCF_SLAVE],
+                           "m:0:0",
+                           5) == -1) {
+                _exit(1);
+            }
+        }
     }
 
     if (!matches.empty()) {
         retval = strdup(matches.back().c_str());
         matches.pop_back();
-
-        fprintf(stderr, "comp gen %s\n", retval);
     }
 
     return retval;
@@ -162,17 +244,14 @@ char **readline_context::attempted_completion(const char *text,
 {
     char **retval = NULL;
 
-    fprintf(stderr, "attempted %s\n", text);
-
     if (loaded_context->rc_possibilities.find("*") !=
         loaded_context->rc_possibilities.end()) {
-        fprintf(stderr, "all poss\n");
         arg_possibilities = &loaded_context->rc_possibilities["*"];
-        rl_completion_append_character = ' ';
+        rl_completion_append_character = loaded_context->rc_append_character;
     }
     else if (start == 0) {
         arg_possibilities = &loaded_context->rc_possibilities["__command"];
-        rl_completion_append_character = ' ';
+        rl_completion_append_character = loaded_context->rc_append_character;
     }
     else {
         char * space;
@@ -180,9 +259,8 @@ char **readline_context::attempted_completion(const char *text,
 
         rl_completion_append_character = 0;
         space = strchr(rl_line_buffer, ' ');
-        assert(space != NULL);
-        cmd = string(rl_line_buffer, 0, space - rl_line_buffer);
-        fprintf(stderr, "cmd %s\n", cmd.c_str());
+        require(space != NULL);
+        cmd = string(rl_line_buffer, space - rl_line_buffer);
 
         vector<string> &proto = loaded_context->rc_prototypes[cmd];
 
@@ -193,11 +271,7 @@ char **readline_context::attempted_completion(const char *text,
             return NULL; /* XXX */
         }
         else {
-            fprintf(stderr, "proto %s\n", proto[0].c_str());
             arg_possibilities = &(loaded_context->rc_possibilities[proto[0]]);
-            fprintf(stderr, "ag %p %d\n",
-                    arg_possibilities,
-                    (int)arg_possibilities->size());
         }
     }
 
@@ -212,12 +286,16 @@ char **readline_context::attempted_completion(const char *text,
 readline_curses::readline_curses()
     : rc_active_context(-1),
       rc_child(-1),
-      rc_value_expiration(0)
+      rc_value_expiration(0),
+      rc_matches_remaining(0),
+      rc_max_match_length(0)
 {
     struct winsize ws;
     int            sp[2];
 
-    if (socketpair(PF_UNIX, SOCK_DGRAM, 0, sp) < 0) {
+    log_info("readline: %s", rl_library_version);
+
+    if (socketpair(PF_UNIX, SOCK_STREAM, 0, sp) < 0) {
         throw error(errno);
     }
 
@@ -260,12 +338,10 @@ readline_curses::readline_curses()
         using_history();
         stifle_history(HISTORY_SIZE);
 
-        /*
-         * XXX Need to keep the input on a single line since the display screws
-         * up if it wraps around.
-         */
-        strcpy(buffer, "set horizontal-scroll-mode on");
-        rl_parse_and_bind(buffer); /* NOTE: buffer is modified */
+        for (int lpc = 0; RL_INIT[lpc]; lpc++) {
+            strcpy(buffer, RL_INIT[lpc]);
+            rl_parse_and_bind(buffer); /* NOTE: buffer is modified */
+        }
 
         child_this = this;
     }
@@ -278,7 +354,7 @@ readline_curses::readline_curses()
 readline_curses::~readline_curses()
 {
     if (this->rc_child == 0) {
-        exit(0);
+        _exit(0);
     }
     else if (this->rc_child > 0) {
         int status;
@@ -292,6 +368,40 @@ readline_curses::~readline_curses()
     }
 }
 
+void readline_curses::store_matches(
+    char **matches, int num_matches, int max_len)
+{
+    char msg[64];
+    int rc;
+
+    max_len = 0;
+    for (int lpc = 0; lpc <= num_matches; lpc++) {
+        max_len = max(max_len, (int)strlen(matches[lpc]));
+    }
+
+    if (last_match_str_valid && strcmp(last_match_str.c_str(), matches[0]) == 0) {
+        if (sendstring(child_this->rc_command_pipe[RCF_SLAVE], "n", 1) == -1) {
+            _exit(1);
+        }
+    }
+    else {
+        rc = snprintf(msg, sizeof(msg), "m:%d:%d", num_matches, max_len);
+        if (sendstring(child_this->rc_command_pipe[RCF_SLAVE], msg, rc) == -1) {
+            _exit(1);
+        }
+        for (int lpc = 1; lpc <= num_matches; lpc++) {
+            if (sendstring(child_this->rc_command_pipe[RCF_SLAVE],
+                           matches[lpc],
+                           strlen(matches[lpc])) == -1) {
+                _exit(1);
+            }
+        }
+
+        last_match_str = matches[0];
+        last_match_str_valid = true;
+    }
+}
+
 void readline_curses::start(void)
 {
     if (this->rc_child != 0) {
@@ -302,9 +412,10 @@ void readline_curses::start(void)
     fd_set rfds;
     int    maxfd;
 
-    assert(!this->rc_contexts.empty());
+    require(!this->rc_contexts.empty());
 
     rl_completer_word_break_characters = (char *)" \t\n"; /* XXX */
+    rl_completion_display_matches_hook = store_matches;
 
     current_context = this->rc_contexts.end();
 
@@ -329,7 +440,10 @@ void readline_curses::start(void)
             if (FD_ISSET(STDIN_FILENO, &ready_rfds)) {
                 struct itimerval itv;
 
-                assert(current_context != this->rc_contexts.end());
+                if (current_context == this->rc_contexts.end()) {
+                    looping = false;
+                    break;
+                }
 
                 itv.it_value.tv_sec     = 0;
                 itv.it_value.tv_usec    = KEY_TIMEOUT;
@@ -343,22 +457,18 @@ void readline_curses::start(void)
                     this->line_ready("");
                     rl_callback_handler_remove();
                 }
-                /* fprintf(stderr, " is done: %d\n", RL_ISSTATE(RL_STATE_DONE)); */
             }
             if (FD_ISSET(this->rc_command_pipe[RCF_SLAVE], &ready_rfds)) {
                 char msg[1024 + 1];
 
-                /* fprintf(stderr, "rl cmd\n"); */
-                if ((rc = read(this->rc_command_pipe[RCF_SLAVE],
-                               msg,
-                               sizeof(msg) - 1)) < 0) {
-                    fprintf(stderr, "read\n");
+                if ((rc = recvstring(this->rc_command_pipe[RCF_SLAVE],
+                                     msg,
+                                     sizeof(msg) - 1)) < 0) {
                 }
                 else {
                     int  context, prompt_start = 0;
                     char type[32];
 
-                    fprintf(stderr, "msg: %s\n", msg);
                     msg[rc] = '\0';
                     if (sscanf(msg, "f:%d:%n", &context, &prompt_start) == 1 &&
                         prompt_start != 0 &&
@@ -367,11 +477,11 @@ void readline_curses::start(void)
                         current_context->second->load();
                         rl_callback_handler_install(&msg[prompt_start],
                                                     line_ready_tramp);
+                        last_match_str_valid = false;
                     }
                     else if (strcmp(msg, "a") == 0) {
                         char reply[4];
 
-                        fprintf(stderr, "abort!!!\n");
                         rl_done = 1;
                         got_timeout = 0;
                         got_line = 1;
@@ -379,11 +489,11 @@ void readline_curses::start(void)
 
                         snprintf(reply, sizeof(reply), "a");
 
-                        if (reliable_send(this->rc_command_pipe[RCF_SLAVE],
-                                          reply,
-                                          strlen(reply)) == -1) {
+                        if (sendstring(this->rc_command_pipe[RCF_SLAVE],
+                                       reply,
+                                       strlen(reply)) == -1) {
                             perror("abort: write failed");
-                            exit(1);
+                            _exit(1);
                         }
                     }
                     else if (sscanf(msg,
@@ -391,7 +501,7 @@ void readline_curses::start(void)
                                     &context,
                                     type,
                                     &prompt_start) == 2) {
-                        assert(this->rc_contexts[context] != NULL);
+                        require(this->rc_contexts[context] != NULL);
 
                         this->rc_contexts[context]->
                         add_possibility(string(type),
@@ -402,7 +512,7 @@ void readline_curses::start(void)
                                     &context,
                                     type,
                                     &prompt_start) == 2) {
-                        assert(this->rc_contexts[context] != NULL);
+                        require(this->rc_contexts[context] != NULL);
 
                         this->rc_contexts[context]->
                         rem_possibility(string(type),
@@ -412,7 +522,7 @@ void readline_curses::start(void)
                         this->rc_contexts[context]->clear_possibilities(type);
                     }
                     else {
-                        fprintf(stderr, "unhandled message: %s\n", msg);
+                        log_error("unhandled message: %s", msg);
                     }
                 }
             }
@@ -421,14 +531,12 @@ void readline_curses::start(void)
         if (got_timeout) {
             char msg[1024];
 
-            fprintf(stderr, "got timeout\n");
             got_timeout = 0;
             snprintf(msg, sizeof(msg), "t:%s", rl_line_buffer);
-            if (reliable_send(this->rc_command_pipe[RCF_SLAVE],
-                              msg,
-                              strlen(msg)) == -1) {
-                perror("got_timeout: write failed");
-                exit(1);
+            if (sendstring(this->rc_command_pipe[RCF_SLAVE],
+                           msg,
+                           strlen(msg)) == -1) {
+                _exit(1);
             }
         }
         if (got_line) {
@@ -440,7 +548,7 @@ void readline_curses::start(void)
             itv.it_interval.tv_sec  = 0;
             itv.it_interval.tv_usec = 0;
             if (setitimer(ITIMER_REAL, &itv, NULL) < 0) {
-                fprintf(stderr, "setitimer");
+                log_error("setitimer: %s", strerror(errno));
             }
             current_context->second->save();
             current_context = this->rc_contexts.end();
@@ -451,13 +559,11 @@ void readline_curses::start(void)
             if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws) == -1) {
                 throw error(errno);
             }
-            fprintf(stderr, "rl winched %d %d\n", ws.ws_row, ws.ws_col);
             got_winch = 0;
             rl_set_screen_size(ws.ws_row, ws.ws_col);
         }
     }
 
-    fprintf(stderr, "writing history\n");
     std::map<int, readline_context *>::iterator citer;
     for (citer = this->rc_contexts.begin();
          citer != this->rc_contexts.end();
@@ -477,7 +583,7 @@ void readline_curses::start(void)
         citer->second->save();
     }
 
-    exit(0);
+    _exit(0);
 }
 
 void readline_curses::line_ready(const char *line)
@@ -509,11 +615,11 @@ void readline_curses::line_ready(const char *line)
         break;
     }
 
-    if (reliable_send(this->rc_command_pipe[RCF_SLAVE],
-                      msg,
-                      strlen(msg)) == -1) {
+    if (sendstring(this->rc_command_pipe[RCF_SLAVE],
+                   msg,
+                   strlen(msg)) == -1) {
         perror("line_ready: write failed");
-        exit(1);
+        _exit(1);
     }
 
     {
@@ -537,51 +643,72 @@ void readline_curses::check_fd_set(fd_set &ready_rfds)
 
         rc = read(this->rc_pty[RCF_MASTER], buffer, sizeof(buffer));
         if (rc > 0) {
-            {
-                int lpc;
-
-                fprintf(stderr, "from child %d|", rc);
-                for (lpc = 0; lpc < rc; lpc++) {
-                    fprintf(stderr, " %d", buffer[lpc]);
-                }
-                fprintf(stderr, "\n");
-            }
-
             this->map_output(buffer, rc);
         }
     }
     if (FD_ISSET(this->rc_command_pipe[RCF_MASTER], &ready_rfds)) {
         char msg[1024 + 1];
 
-        rc = read(this->rc_command_pipe[RCF_MASTER], msg, sizeof(msg) - 1);
-        if (rc >= 1) {
+        rc = recvstring(this->rc_command_pipe[RCF_MASTER], msg, sizeof(msg) - 1);
+        if (rc >= 0) {
             string old_value = this->rc_value;
 
             msg[rc] = '\0';
-            fprintf(stderr, "child command: %s\n", msg);
-            this->rc_value = string(&msg[2]);
-            switch (msg[0]) {
-            case 'a':
-                fprintf(stderr, "aborted!\n");
-                this->rc_active_context = -1;
-                this->vc_past_lines.clear();
-                this->rc_abort.invoke(this);
-                curs_set(0);
-                break;
-
-            case 't':
-                if (this->rc_value != old_value) {
-                    this->rc_timeout.invoke(this);
+            if (this->rc_matches_remaining > 0) {
+                this->rc_matches.push_back(msg);
+                this->rc_matches_remaining -= 1;
+                if (this->rc_matches_remaining == 0) {
+                    this->rc_display_match.invoke(this);
                 }
-                break;
+            }
+            else if (msg[0] == 'm') {
+                if (sscanf(msg, "m:%d:%d", &this->rc_matches_remaining,
+                           &this->rc_max_match_length) != 2) {
+                    require(0);
+                }
+                this->rc_matches.clear();
+                if (this->rc_matches_remaining == 0) {
+                    this->rc_display_match.invoke(this);
+                }
+            }
+            else {
+                switch (msg[0]) {
+                case 't':
+                case 'd':
+                    this->rc_value = string(&msg[2]);
+                    break;
+                }
+                switch (msg[0]) {
+                case 'a':
+                    require(rc == 1);
 
-            case 'd':
-                fprintf(stderr, "done!\n");
-                this->rc_active_context = -1;
-                this->vc_past_lines.clear();
-                this->rc_perform.invoke(this);
-                curs_set(0);
-                break;
+                    this->rc_active_context = -1;
+                    this->vc_past_lines.clear();
+                    this->rc_matches.clear();
+                    this->rc_abort.invoke(this);
+                    this->rc_display_match.invoke(this);
+                    curs_set(0);
+                    break;
+
+                case 't':
+                    if (this->rc_value != old_value) {
+                        this->rc_timeout.invoke(this);
+                    }
+                    break;
+
+                case 'd':
+                    this->rc_active_context = -1;
+                    this->vc_past_lines.clear();
+                    this->rc_matches.clear();
+                    this->rc_perform.invoke(this);
+                    this->rc_display_match.invoke(this);
+                    curs_set(0);
+                    break;
+
+                case 'n':
+                    this->rc_display_next.invoke(this);
+                    break;
+                }
             }
         }
     }
@@ -596,7 +723,6 @@ void readline_curses::handle_key(int ch)
     if (write(this->rc_pty[RCF_MASTER], bch, len) == -1) {
         perror("handle_key: write failed");
     }
-    fprintf(stderr, "to child %d\n", bch[0]);
     if (ch == '\t' || ch == '\r') {
         this->vc_past_lines.clear();
     }
@@ -611,9 +737,9 @@ void readline_curses::focus(int context, const char *prompt)
     this->rc_active_context = context;
 
     snprintf(buffer, sizeof(buffer), "f:%d:%s", context, prompt);
-    if (reliable_send(this->rc_command_pipe[RCF_MASTER],
-                      buffer,
-                      strlen(buffer) + 1) == -1) {
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
         perror("focus: write failed");
     }
     wmove(this->vc_window, this->get_actual_y(), 0);
@@ -625,9 +751,9 @@ void readline_curses::abort()
     char buffer[1024];
 
     snprintf(buffer, sizeof(buffer), "a");
-    if (reliable_send(this->rc_command_pipe[RCF_MASTER],
-                      buffer,
-                      strlen(buffer) + 1) == -1) {
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer)) == -1) {
         perror("abort: write failed");
     }
 }
@@ -638,13 +764,16 @@ void readline_curses::add_possibility(int context,
 {
     char buffer[1024];
 
+    if (value.empty()) {
+        return;
+    }
+
     snprintf(buffer, sizeof(buffer),
              "ap:%d:%s:%s",
              context, type.c_str(), value.c_str());
-    fprintf(stderr, "msg: %s\n", buffer);
-    if (reliable_send(this->rc_command_pipe[RCF_MASTER],
-                      buffer,
-                      strlen(buffer) + 1) == -1) {
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
         perror("add_possibility: write failed");
     }
 }
@@ -658,9 +787,9 @@ void readline_curses::rem_possibility(int context,
     snprintf(buffer, sizeof(buffer),
              "rp:%d:%s:%s",
              context, type.c_str(), value.c_str());
-    if (reliable_send(this->rc_command_pipe[RCF_MASTER],
-                      buffer,
-                      strlen(buffer) + 1) == -1) {
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
         perror("rem_possiblity: write failed");
     }
 }
@@ -672,9 +801,9 @@ void readline_curses::clear_possibilities(int context, string type)
     snprintf(buffer, sizeof(buffer),
              "cp:%d:%s",
              context, type.c_str());
-    if (reliable_send(this->rc_command_pipe[RCF_MASTER],
-                      buffer,
-                      strlen(buffer) + 1) == -1) {
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
         perror("clear_possiblity: write failed");
     }
 }
@@ -721,4 +850,28 @@ void readline_curses::do_update(void)
         this->set_x(0);
     }
     vt52_curses::do_update();
+
+    if (this->rc_active_context != -1) {
+        readline_context *rc = this->rc_contexts[this->rc_active_context];
+        readline_highlighter_t hl = rc->get_highlighter();
+
+        if (hl != NULL) {
+            char line[1024];
+            attr_line_t al;
+            string &str = al.get_string();
+            int rc;
+
+            rc = mvwinnstr(this->vc_window,
+                           this->get_actual_y(), 0,
+                           line, sizeof(line));
+
+            str = string(line, rc);
+
+            hl(al, this->vc_x);
+            view_curses::mvwattrline(this->vc_window,
+                this->get_actual_y(), 0, al, line_range(0, rc));
+
+            wmove(this->vc_window, this->get_actual_y(), this->vc_x);
+        }
+    }
 }
