@@ -37,7 +37,7 @@
 #include <sys/types.h>
 #include <dirent.h>
 
-#include <openssl/sha.h>
+#include "spookyhash/SpookyV2.h"
 
 #include <algorithm>
 
@@ -89,7 +89,7 @@ typedef std::vector<std::pair<int, string> > timestamped_list_t;
 static std::vector<content_line_t> marked_session_lines;
 static std::vector<content_line_t> offset_session_lines;
 
-static void bind_line(sqlite3 *db,
+static bool bind_line(sqlite3 *db,
                       sqlite3_stmt *stmt,
                       content_line_t cl,
                       time_t session_time)
@@ -99,18 +99,23 @@ static void bind_line(sqlite3 *db,
     logfile *lf;
 
     lf = lss.find(cl);
+
+    if (lf == NULL) {
+        return false;
+    }
+
     line_iter = lf->begin() + cl;
 
     char timestamp[64];
 
     sql_strftime(timestamp, sizeof(timestamp),
-                 lf->original_line_time(line_iter));
+                 lf->original_line_time(line_iter), 'T');
     if (sqlite3_bind_text(stmt, 1,
                           timestamp, strlen(timestamp),
                           SQLITE_TRANSIENT) != SQLITE_OK) {
         fprintf(stderr, "error: could not bind log time -- %s\n",
                 sqlite3_errmsg(db));
-        return;
+        return false;
     }
 
     const std::string &format_name = lf->get_format()->get_name();
@@ -120,7 +125,7 @@ static void bind_line(sqlite3 *db,
                           SQLITE_TRANSIENT) != SQLITE_OK) {
         fprintf(stderr, "error: could not bind log format -- %s\n",
                     sqlite3_errmsg(db));
-        return;
+        return false;
     }
 
     std::string line_hash = hash_string(lf->read_line(line_iter));
@@ -130,14 +135,16 @@ static void bind_line(sqlite3 *db,
                           SQLITE_TRANSIENT) != SQLITE_OK) {
         fprintf(stderr, "error: could not bind log hash -- %s\n",
                 sqlite3_errmsg(db));
-        return;
+        return false;
     }
 
     if (sqlite3_bind_int64(stmt, 4, session_time) != SQLITE_OK) {
         fprintf(stderr, "error: could not bind session time -- %s\n",
                 sqlite3_errmsg(db));
-        return;
+        return false;
     }
+
+    return true;
 }
 
 struct session_file_info {
@@ -243,19 +250,22 @@ static void cleanup_session_data(void)
 
 void init_session(void)
 {
-    byte_array<SHA_DIGEST_LENGTH> hash;
-    SHA_CTX context;
+    byte_array<2, uint64> hash;
+    SpookyHash context;
 
     lnav_data.ld_session_time = time(NULL);
 
-    SHA_Init(&context);
-    sha_updater updater(&context);
+    context.Init(0, 0);
+    hash_updater updater(&context);
     for_each(lnav_data.ld_file_names.begin(),
              lnav_data.ld_file_names.end(),
              object_field(updater, &pair<string, int>::first));
-    SHA_Final(hash.out(), &context);
+    context.Final(hash.out(0), hash.out(1));
 
     lnav_data.ld_session_id = hash.to_string();
+
+    log_info("init_session: time=%d; id=%s", lnav_data.ld_session_time,
+        lnav_data.ld_session_id.c_str());
 }
 
 void scan_sessions(void)
@@ -342,7 +352,7 @@ static void load_time_bookmarks(void)
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
     logfile_sub_source::iterator file_iter;
 
-    fprintf(stderr, "load time bookmarks\n");
+    log_info("loading bookmark db: %s", db_path.c_str());
 
     if (sqlite3_open(db_path.c_str(), db.out()) != SQLITE_OK) {
         return;
@@ -373,12 +383,10 @@ static void load_time_bookmarks(void)
 
         base_content_line = lss.get_file_base_content_line(file_iter);
 
-        fprintf(stderr, "checking bookmarks for %s\n", lf->get_filename().c_str());
-
         logfile::iterator line_iter = lf->begin();
 
         sql_strftime(low_timestamp, sizeof(low_timestamp),
-                     lf->original_line_time(line_iter));
+                     lf->original_line_time(line_iter), 'T');
 
         if (sqlite3_bind_int64(stmt.in(), 1, lnav_data.ld_session_load_time) != SQLITE_OK) {
             fprintf(stderr, "error: could not bind session time -- %s\n",
@@ -397,7 +405,7 @@ static void load_time_bookmarks(void)
         line_iter = lf->end();
         --line_iter;
         sql_strftime(high_timestamp, sizeof(high_timestamp),
-                     lf->original_line_time(line_iter));
+                     lf->original_line_time(line_iter), 'T');
 
         if (sqlite3_bind_text(stmt.in(), 3,
                               high_timestamp, strlen(high_timestamp),
@@ -420,9 +428,6 @@ static void load_time_bookmarks(void)
         date_time_scanner dts;
         bool done = false;
         string line;
-
-        fprintf(stderr, "time range %s %s\n", low_timestamp, high_timestamp);
-
         int64_t last_mark_time = -1;
 
         while (!done) {
@@ -476,11 +481,17 @@ static void load_time_bookmarks(void)
                         content_line_t line_cl = content_line_t(
                             base_content_line + std::distance(lf->begin(), line_iter));
 
-                        marked_session_lines.push_back(line_cl);
-                        lss.set_user_mark(&textview_curses::BM_USER, line_cl);
                         if (part_name != NULL && part_name[0] != '\0') {
+                            lss.set_user_mark(&textview_curses::BM_PARTITION,
+                                line_cl);
                             bm_meta[line_cl].bm_name = part_name;
                         }
+                        else {
+                            marked_session_lines.push_back(line_cl);
+                            lss.set_user_mark(&textview_curses::BM_USER,
+                                line_cl);
+
+                        }
                     }
 
                     ++line_iter;
@@ -531,12 +542,10 @@ static void load_time_bookmarks(void)
 
         lss.find(lf->get_filename().c_str(), base_content_line);
 
-        fprintf(stderr, "checking time offsets for %s\n", lf->get_filename().c_str());
-
         logfile::iterator line_iter = lf->begin();
 
         sql_strftime(low_timestamp, sizeof(low_timestamp),
-                     lf->original_line_time(line_iter));
+                     lf->original_line_time(line_iter), 'T');
 
         if (sqlite3_bind_int64(stmt.in(), 1, lnav_data.ld_session_load_time) != SQLITE_OK) {
             fprintf(stderr, "error: could not bind session time -- %s\n",
@@ -555,7 +564,7 @@ static void load_time_bookmarks(void)
         line_iter = lf->end();
         --line_iter;
         sql_strftime(high_timestamp, sizeof(high_timestamp),
-                     lf->original_line_time(line_iter));
+                     lf->original_line_time(line_iter), 'T');
 
         if (sqlite3_bind_text(stmt.in(), 3,
                               high_timestamp, strlen(high_timestamp),
@@ -578,9 +587,6 @@ static void load_time_bookmarks(void)
         date_time_scanner dts;
         bool done = false;
         string line;
-
-        fprintf(stderr, "time range %s %s\n", low_timestamp, high_timestamp);
-
         int64_t last_mark_time = -1;
 
         while (!done) {
@@ -672,6 +678,13 @@ static int read_save_time(yajlpp_parse_context *ypc, long long value)
     return 1;
 }
 
+static int read_time_offset(yajlpp_parse_context *ypc, int value)
+{
+    lnav_data.ld_log_source.set_time_offset(value);
+
+    return 1;
+}
+
 static int read_files(yajlpp_parse_context *ypc, const unsigned char *str, size_t len)
 {
     return 1;
@@ -725,10 +738,12 @@ static int read_word_wrap(yajlpp_parse_context *ypc, int value)
                      lnav_view_strings + LNV__MAX,
                      ypc->get_path_fragment(-2));
     view_index = view_name - lnav_view_strings;
-    if (view_index < LNV__MAX) {
+    if (view_index == LNV_HELP) {
+
+    }
+    else if (view_index < LNV__MAX) {
         textview_curses &tc = lnav_data.ld_views[view_index];
 
-        fprintf(stderr, "set word wrap %d\n", value);
         tc.set_word_wrap(value);
     }
 
@@ -746,6 +761,7 @@ static int read_commands(yajlpp_parse_context *ypc, const unsigned char *str, si
 
 static struct json_path_handler view_info_handlers[] = {
     json_path_handler("/save-time",               read_save_time),
+    json_path_handler("/time-offset",             read_time_offset),
     json_path_handler("/files#",                  read_files),
     json_path_handler("/views/([^/]+)/top_line",  read_top_line),
     json_path_handler("/views/([^/]+)/search",    read_last_search),
@@ -784,6 +800,7 @@ void load_session(void)
         unsigned char buffer[1024];
         size_t        rc;
 
+        log_info("loading session file: %s", view_info_name.c_str());
         while ((rc = read(fd, buffer, sizeof(buffer))) > 0) {
             yajl_parse(handle, buffer, rc);
         }
@@ -799,6 +816,57 @@ static void yajl_writer(void *context, const char *str, size_t len)
     fwrite(str, len, 1, file);
 }
 
+static void save_user_bookmarks(
+    sqlite3 *db, sqlite3_stmt *stmt, bookmark_vector<content_line_t> &user_marks)
+{
+    logfile_sub_source &lss = lnav_data.ld_log_source;
+    std::map<content_line_t, bookmark_metadata> &bm_meta =
+        lss.get_user_bookmark_metadata();
+    bookmark_vector<content_line_t>::iterator iter;
+
+    for (iter = user_marks.begin(); iter != user_marks.end(); ++iter) {
+        std::map<content_line_t, bookmark_metadata>::iterator meta_iter;
+        logfile::iterator line_iter;
+        content_line_t cl = *iter;
+
+        meta_iter = bm_meta.find(cl);
+
+        marked_session_lines.push_back(cl);
+
+        if (!bind_line(db, stmt, cl, lnav_data.ld_session_time)) {
+            continue;
+        }
+
+        if (meta_iter == bm_meta.end()) {
+            if (sqlite3_bind_text(stmt, 5, "", 0, SQLITE_TRANSIENT) != SQLITE_OK) {
+                fprintf(stderr, "error: could not bind log hash -- %s\n",
+                        sqlite3_errmsg(db));
+                return;
+            }
+        }
+        else {
+            if (sqlite3_bind_text(stmt, 5,
+                                  meta_iter->second.bm_name.c_str(),
+                                  meta_iter->second.bm_name.length(),
+                                  SQLITE_TRANSIENT) != SQLITE_OK) {
+                fprintf(stderr, "error: could not bind log hash -- %s\n",
+                        sqlite3_errmsg(db));
+                return;
+            }
+        }
+
+        if (sqlite3_step(stmt) != SQLITE_DONE) {
+            fprintf(stderr,
+                    "error: could not execute bookmark insert statement -- %s\n",
+                    sqlite3_errmsg(db));
+            return;
+        }
+
+        sqlite3_reset(stmt);
+    }
+
+}
+
 static void save_time_bookmarks(void)
 {
     auto_mem<sqlite3, sqlite_close_wrapper> db;
@@ -822,10 +890,7 @@ static void save_time_bookmarks(void)
     }
 
     logfile_sub_source &lss = lnav_data.ld_log_source;
-    std::map<content_line_t, bookmark_metadata> &bm_meta = lss.get_user_bookmark_metadata();
     bookmarks<content_line_t>::type &bm = lss.get_user_bookmarks();
-    bookmark_vector<content_line_t> &user_marks = bm[&textview_curses::BM_USER];
-    bookmark_vector<content_line_t>::iterator iter;
 
     if (sqlite3_prepare_v2(db.in(),
                            "DELETE FROM bookmarks WHERE "
@@ -843,7 +908,10 @@ static void save_time_bookmarks(void)
     for (std::vector<content_line_t>::iterator cl_iter = marked_session_lines.begin();
          cl_iter != marked_session_lines.end();
          ++cl_iter) {
-        bind_line(db.in(), stmt.in(), *cl_iter, lnav_data.ld_session_time);
+        if (!bind_line(
+            db.in(), stmt.in(), *cl_iter, lnav_data.ld_session_time)) {
+            continue;
+        }
 
         if (sqlite3_step(stmt.in()) != SQLITE_DONE) {
             fprintf(stderr,
@@ -883,9 +951,13 @@ static void save_time_bookmarks(void)
                 continue;
 
             base_content_line = lss.get_file_base_content_line(file_iter);
-            base_content_line = content_line_t(base_content_line + lf->size() - 1);
+            base_content_line = content_line_t(
+                base_content_line + lf->size() - 1);
 
-            bind_line(db.in(), stmt.in(), base_content_line, lnav_data.ld_session_time);
+            if (!bind_line(db.in(), stmt.in(), base_content_line,
+                lnav_data.ld_session_time)) {
+                continue;
+            }
 
             if (sqlite3_bind_null(stmt.in(), 5) != SQLITE_OK) {
                 fprintf(stderr, "error: could not bind log hash -- %s\n",
@@ -904,44 +976,8 @@ static void save_time_bookmarks(void)
         }
     }
 
-    for (iter = user_marks.begin(); iter != user_marks.end(); ++iter) {
-        std::map<content_line_t, bookmark_metadata>::iterator meta_iter;
-        logfile::iterator line_iter;
-        content_line_t cl = *iter;
-
-        meta_iter = bm_meta.find(cl);
-
-        marked_session_lines.push_back(cl);
-
-        bind_line(db.in(), stmt.in(), cl, lnav_data.ld_session_time);
-
-        if (meta_iter == bm_meta.end()) {
-            if (sqlite3_bind_text(stmt.in(), 5, "", 0, SQLITE_TRANSIENT) != SQLITE_OK) {
-                fprintf(stderr, "error: could not bind log hash -- %s\n",
-                        sqlite3_errmsg(db.in()));
-                return;
-            }
-        }
-        else {
-            if (sqlite3_bind_text(stmt.in(), 5,
-                                  meta_iter->second.bm_name.c_str(),
-                                  meta_iter->second.bm_name.length(),
-                                  SQLITE_TRANSIENT) != SQLITE_OK) {
-                fprintf(stderr, "error: could not bind log hash -- %s\n",
-                        sqlite3_errmsg(db.in()));
-                return;
-            }
-        }
-
-        if (sqlite3_step(stmt.in()) != SQLITE_DONE) {
-            fprintf(stderr,
-                    "error: could not execute bookmark insert statement -- %s\n",
-                    sqlite3_errmsg(db));
-            return;
-        }
-
-        sqlite3_reset(stmt.in());
-    }
+    save_user_bookmarks(db.in(), stmt.in(), bm[&textview_curses::BM_USER]);
+    save_user_bookmarks(db.in(), stmt.in(), bm[&textview_curses::BM_PARTITION]);
 
     if (sqlite3_prepare_v2(db.in(),
                            "DELETE FROM time_offset WHERE "
@@ -951,7 +987,7 @@ static void save_time_bookmarks(void)
                            stmt.out(),
                            NULL) != SQLITE_OK) {
         fprintf(stderr,
-                "error: could not prepare bookmark replace statemnt -- %s\n",
+                "error: could not prepare time_offset delete statemnt -- %s\n",
                 sqlite3_errmsg(db));
         return;
     }
@@ -959,7 +995,10 @@ static void save_time_bookmarks(void)
     for (std::vector<content_line_t>::iterator cl_iter = offset_session_lines.begin();
          cl_iter != offset_session_lines.end();
          ++cl_iter) {
-        bind_line(db.in(), stmt.in(), *cl_iter, lnav_data.ld_session_time);
+        if (!bind_line(
+            db.in(), stmt.in(), *cl_iter, lnav_data.ld_session_time)) {
+            continue;
+        }
 
         if (sqlite3_step(stmt.in()) != SQLITE_DONE) {
             fprintf(stderr,
@@ -1000,7 +1039,10 @@ static void save_time_bookmarks(void)
 
             base_content_line = lss.get_file_base_content_line(file_iter);
 
-            bind_line(db.in(), stmt.in(), base_content_line, lnav_data.ld_session_time);
+            if (!bind_line(db.in(), stmt.in(), base_content_line,
+                lnav_data.ld_session_time)) {
+                continue;
+            }
 
             if (sqlite3_bind_null(stmt.in(), 5) != SQLITE_OK) {
                 fprintf(stderr, "error: could not bind log hash -- %s\n",
@@ -1043,7 +1085,7 @@ static void save_time_bookmarks(void)
         char timestamp[64];
 
         sql_strftime(timestamp, sizeof(timestamp),
-                     lf->original_line_time(line_iter));
+                     lf->original_line_time(line_iter), 'T');
         if (sqlite3_bind_text(stmt.in(), 1,
                               timestamp, strlen(timestamp),
                               SQLITE_TRANSIENT) != SQLITE_OK) {
@@ -1151,6 +1193,9 @@ void save_session(void)
             root_map.gen("save-time");
             root_map.gen((long long)time(NULL));
 
+            root_map.gen("time-offset");
+            root_map.gen(lnav_data.ld_log_source.is_time_offset_enabled());
+
             root_map.gen("files");
 
             {
@@ -1244,6 +1289,8 @@ void reset_session(void)
     textview_curses::highlight_map_t::iterator hl_iter = hmap.begin();
     logfile_sub_source &lss = lnav_data.ld_log_source;
 
+    log_info("reset session: time=%d", lnav_data.ld_session_time);
+
     save_session();
     scan_sessions();
 
@@ -1271,4 +1318,5 @@ void reset_session(void)
     }
 
     lss.get_user_bookmarks()[&textview_curses::BM_USER].clear();
+    lss.get_user_bookmarks()[&textview_curses::BM_PARTITION].clear();
 }
