@@ -37,6 +37,7 @@
 
 #include "auto_mem.hh"
 #include "sql_util.hh"
+#include "lnav_log.hh"
 
 /**
  * Copied from -- http://www.sqlite.org/lang_keywords.html
@@ -212,6 +213,13 @@ const char *sql_function_names[] = {
     "upper",
     "zeroblob",
 
+    /* http://www.sqlite.org/lang_datefunc.html */
+    "date",
+    "time",
+    "datetime",
+    "julianday",
+    "strftime",
+
     NULL
 };
 
@@ -260,9 +268,7 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
                           &smc,
                           errmsg.out());
     if (retval != SQLITE_OK) {
-        fprintf(stderr,
-                "error: could not get collation list -- %s\n",
-                errmsg.in());
+        log_error("could not get collation list -- %s", errmsg.in());
         return retval;
     }
 
@@ -272,7 +278,7 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
                           &smc,
                           errmsg.out());
     if (retval != SQLITE_OK) {
-        fprintf(stderr, "error: could not get DB list -- %s\n", errmsg.in());
+        log_error("could not get DB list -- %s", errmsg.in());
         return retval;
     }
 
@@ -282,8 +288,8 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
         struct table_list_data       tld = { &smc, &iter };
         auto_mem<char, sqlite3_free> query;
 
-        query = sqlite3_mprintf("SELECT name FROM %Q.sqlite_master "
-                                "WHERE type='table'",
+        query = sqlite3_mprintf("SELECT name,sql FROM %Q.sqlite_master "
+                                "WHERE type in ('table', 'view')",
                                 iter->first.c_str());
 
         retval = sqlite3_exec(db,
@@ -292,9 +298,7 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
                               &tld,
                               errmsg.out());
         if (retval != SQLITE_OK) {
-            fprintf(stderr,
-                    "error: could not get table list -- %s\n",
-                    errmsg.in());
+            log_error("could not get table list -- %s", errmsg.in());
             return retval;
         }
 
@@ -315,12 +319,10 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
             retval = sqlite3_exec(db,
                                   table_query,
                                   smc.smc_table_info,
-                                  &table_name,
+                                  &smc,
                                   errmsg.out());
             if (retval != SQLITE_OK) {
-                fprintf(stderr,
-                        "error: could not get table info -- %s\n",
-                        errmsg.in());
+                log_error("could not get table info -- %s", errmsg.in());
                 return retval;
             }
 
@@ -335,12 +337,10 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
             retval = sqlite3_exec(db,
                                   table_query,
                                   smc.smc_foreign_key_list,
-                                  &table_name,
+                                  &smc,
                                   errmsg.out());
             if (retval != SQLITE_OK) {
-                fprintf(stderr,
-                        "error: could not get foreign key list -- %s\n",
-                        errmsg.in());
+                log_error("could not get foreign key list -- %s", errmsg.in());
                 return retval;
             }
         }
@@ -349,6 +349,77 @@ int walk_sqlite_metadata(sqlite3 *db, struct sqlite_metadata_callbacks &smc)
     return retval;
 }
 
+static int schema_collation_list(void *ptr,
+                                 int ncols,
+                                 char **colvalues,
+                                 char **colnames)
+{
+    return 0;
+}
+
+static int schema_db_list(void *ptr,
+                          int ncols,
+                          char **colvalues,
+                          char **colnames)
+{
+    struct sqlite_metadata_callbacks *smc = (sqlite_metadata_callbacks *)ptr;
+    string &schema_out = *((string *)smc->smc_userdata);
+    auto_mem<char, sqlite3_free> attach_sql;
+
+    attach_sql = sqlite3_mprintf("ATTACH DATABASE %Q AS %Q;\n",
+        colvalues[2], colvalues[1]);
+
+    schema_out += attach_sql;
+
+    return 0;
+}
+
+static int schema_table_list(void *ptr,
+                             int ncols,
+                             char **colvalues,
+                             char **colnames)
+{
+    struct sqlite_metadata_callbacks *smc = (sqlite_metadata_callbacks *)ptr;
+    string &schema_out = *((string *)smc->smc_userdata);
+    auto_mem<char, sqlite3_free> create_sql;
+
+    create_sql = sqlite3_mprintf("%s;\n", colvalues[1]);
+
+    schema_out += create_sql;
+
+    return 0;
+}
+
+static int schema_table_info(void *ptr,
+                             int ncols,
+                             char **colvalues,
+                             char **colnames)
+{
+    return 0;
+}
+
+static int schema_foreign_key_list(void *ptr,
+                                   int ncols,
+                                   char **colvalues,
+                                   char **colnames)
+{
+    return 0;
+}
+
+void dump_sqlite_schema(sqlite3 *db, std::string &schema_out)
+{
+    struct sqlite_metadata_callbacks schema_sql_meta_callbacks = {
+        schema_collation_list,
+        schema_db_list,
+        schema_table_list,
+        schema_table_info,
+        schema_foreign_key_list,
+        &schema_out
+    };
+
+    walk_sqlite_metadata(db, schema_sql_meta_callbacks);
+}
+
 void attach_sqlite_db(sqlite3 *db, const std::string &filename)
 {
     static pcrecpp::RE db_name_converter("[^\\w]");
@@ -360,18 +431,16 @@ void attach_sqlite_db(sqlite3 *db, const std::string &filename)
                            -1,
                            stmt.out(),
                            NULL) != SQLITE_OK) {
-        fprintf(stderr,
-                "error: could not prepare DB attach statement -- %s\n",
-                sqlite3_errmsg(db));
+        log_error("could not prepare DB attach statement -- %s",
+            sqlite3_errmsg(db));
         return;
     }
 
     if (sqlite3_bind_text(stmt.in(), 1,
                           filename.c_str(), filename.length(),
                           SQLITE_TRANSIENT) != SQLITE_OK) {
-        fprintf(stderr,
-                "error: could not bind DB attach statement -- %s\n",
-                sqlite3_errmsg(db));
+        log_error("could not bind DB attach statement -- %s",
+            sqlite3_errmsg(db));
         return;
     }
 
@@ -390,30 +459,30 @@ void attach_sqlite_db(sqlite3 *db, const std::string &filename)
     if (sqlite3_bind_text(stmt.in(), 2,
                           db_name.c_str(), db_name.length(),
                           SQLITE_TRANSIENT) != SQLITE_OK) {
-        fprintf(stderr,
-                "error: could not bind DB attach statement -- %s\n",
-                sqlite3_errmsg(db));
+        log_error("could not bind DB attach statement -- %s",
+            sqlite3_errmsg(db));
         return;
     }
 
     if (sqlite3_step(stmt.in()) != SQLITE_DONE) {
-        fprintf(stderr,
-                "error: could not execute DB attach statement -- %s\n",
-                sqlite3_errmsg(db));
+        log_error("could not execute DB attach statement -- %s",
+            sqlite3_errmsg(db));
         return;
     }
 }
 
-void sql_strftime(char *buffer, size_t buffer_size, time_t time, int millis)
+void sql_strftime(char *buffer, size_t buffer_size, time_t time, int millis,
+    char sep)
 {
     struct tm gmtm;
 
     gmtime_r(&time, &gmtm);
     snprintf(buffer, buffer_size,
-             "%4d-%02d-%02dT%02d:%02d:%02d.%03d",
+             "%4d-%02d-%02d%c%02d:%02d:%02d.%03d",
              gmtm.tm_year + 1900,
              gmtm.tm_mon + 1,
              gmtm.tm_mday,
+             sep,
              gmtm.tm_hour,
              gmtm.tm_min,
              gmtm.tm_sec,
