@@ -41,8 +41,11 @@
 using namespace std;
 
 bookmark_type_t textview_curses::BM_USER;
+bookmark_type_t textview_curses::BM_PARTITION;
 bookmark_type_t textview_curses::BM_SEARCH;
 
+string_attr_type textview_curses::SA_BODY;
+
 textview_curses::textview_curses()
     : tc_sub_source(NULL),
       tc_delegate(NULL),
@@ -99,6 +102,7 @@ void textview_curses::listview_value_for_row(const listview_curses &lv,
                                              attr_line_t &value_out)
 {
     bookmark_vector<vis_line_t> &user_marks = this->tc_bookmarks[&BM_USER];
+    bookmark_vector<vis_line_t> &part_marks = this->tc_bookmarks[&BM_PARTITION];
     string_attrs_t &             sa         = value_out.get_attrs();
     string &                     str        = value_out.get_string();
     highlight_map_t::iterator    iter;
@@ -111,7 +115,7 @@ void textview_curses::listview_value_for_row(const listview_curses &lv,
 
     struct line_range body;
 
-    body = find_string_attr_range(sa, "body");
+    body = find_string_attr_range(sa, &SA_BODY);
     if (body.lr_start == -1) {
         body.lr_start = 0;
         body.lr_end = str.size();
@@ -125,8 +129,8 @@ void textview_curses::listview_value_for_row(const listview_curses &lv,
         int off, hcount = 0;
         size_t re_end;
 
-        if (body.lr_end > 2048)
-            re_end = 2048;
+        if (body.lr_end > 8192)
+            re_end = 8192;
         else
             re_end = body.lr_end;
 
@@ -153,8 +157,8 @@ void textview_curses::listview_value_for_row(const listview_curses &lv,
                 }
 
                 if (lr.lr_end > lr.lr_start) {
-                    sa[lr].insert(make_string_attr("style", iter->second.
-                                                   get_attrs(hcount)));
+                    sa.push_back(string_attr(
+                        lr, &view_curses::VC_STYLE, iter->second.get_attrs(hcount)));
                     hcount++;
 
                     off = matches[1];
@@ -225,22 +229,24 @@ void textview_curses::listview_value_for_row(const listview_curses &lv,
         string_attrs_t::iterator iter;
 
         for (iter = sa.begin(); iter != sa.end(); iter++) {
-            attrs_map_t &         am = iter->second;
-            attrs_map_t::iterator am_iter;
-
-            for (am_iter = am.begin(); am_iter != am.end(); am_iter++) {
-                if (am_iter->first == "style") {
-                    am_iter->second.sa_int ^= A_REVERSE;
-                }
+            if (iter->sa_type == &view_curses::VC_STYLE) {
+                iter->sa_value.sav_int ^= A_REVERSE;
             }
         }
 
-        sa[lr].insert(make_string_attr("style", A_REVERSE));
+        sa.push_back(string_attr(lr, &view_curses::VC_STYLE, A_REVERSE));
+    }
+    else if (binary_search(part_marks.begin(), part_marks.end(), row + 1)) {
+        sa.push_back(string_attr(
+            line_range(0), &view_curses::VC_STYLE, A_UNDERLINE));
     }
 }
 
 bool textview_curses::handle_mouse(mouse_event &me)
 {
+    unsigned long width;
+    vis_line_t height;
+
     if (this->tc_selection_start == -1 &&
         listview_curses::handle_mouse(me)) {
         return true;
@@ -261,6 +267,8 @@ bool textview_curses::handle_mouse(mouse_event &me)
         mouse_line = this->get_bottom();
     }
 
+    this->get_dimensions(height, width);
+
     switch (me.me_state) {
     case BUTTON_STATE_PRESSED:
         this->tc_selection_start = mouse_line;
@@ -273,6 +281,11 @@ bool textview_curses::handle_mouse(mouse_event &me)
             me.me_y = 0;
             mouse_line = this->get_top();
         }
+        if (me.me_y >= height && this->get_top() < this->get_top_for_last_row()) {
+            this->shift_top(vis_line_t(1));
+            me.me_y = height;
+            mouse_line = this->get_bottom();
+        }
 
         if (this->tc_selection_last == mouse_line)
             break;
