@@ -36,6 +36,7 @@
 #include <iostream>
 
 #include "pcrepp.hh"
+#include "textview_curses.hh"
 #include "data_scanner.hh"
 #include "data_parser.hh"
 #include "log_format.hh"
@@ -51,9 +52,9 @@ int main(int argc, char *argv[])
     bool prompt = false;
 
     {
-        std::vector<std::string> errors;
+        std::vector<std::string> paths, errors;
 
-        load_formats(errors);
+        load_formats(paths, errors);
     }
 
     while ((c = getopt(argc, argv, "p")) != -1) {
@@ -132,13 +133,16 @@ int main(int argc, char *argv[])
 
                 string sub_line = line.substr(13);
                 struct line_range body(0, sub_line.length());
+                shared_buffer share_manager;
+                shared_buffer_ref sbr;
 
+                sbr.share(share_manager, (char *)sub_line.c_str(), sub_line.size());
                 if (format.get() != NULL) {
                     vector<logline_value> ll_values;
                     string_attrs_t sa;
 
-                    format->annotate(sub_line, sa, ll_values);
-                    body = find_string_attr_range(sa, "body");
+                    format->annotate(sbr, sa, ll_values);
+                    body = find_string_attr_range(sa, &textview_curses::SA_BODY);
                 }
 
                 data_parser::TRACE_FILE = fopen("scanned.dpt", "w");
