@@ -21,7 +21,7 @@ run_test ./drive_logfile ${srcdir}/logfile_empty.0
 
 on_error_fail_with "Didn't handle empty log?"
 
-
+touch -t 200711030923 ${srcdir}/logfile_syslog.0
 run_test ./drive_logfile -t -f syslog_log ${srcdir}/logfile_syslog.0
 
 check_output "Syslog timestamp interpreted incorrectly?" <<EOF
@@ -31,6 +31,7 @@ Nov 03 09:23:38 2007 -- 000
 Nov 03 09:47:02 2007 -- 000
 EOF
 
+touch -t 200711030923 ${srcdir}/logfile_syslog.1
 run_test ./drive_logfile -t -f syslog_log ${srcdir}/logfile_syslog.1
 
 check_output "Syslog timestamp interpreted incorrectly for year end?" <<EOF
@@ -40,13 +41,40 @@ Dec 03 09:23:38 2006 -- 000
 Jan 03 09:47:02 2007 -- 000
 EOF
 
+gzip -c ${srcdir}/logfile_syslog.1 > logfile_syslog.1.gz
+
+run_test ./drive_logfile -t -f syslog_log logfile_syslog.1.gz
+
+check_output "Syslog timestamp incorrect for gzipped file?" <<EOF
+Dec 03 09:23:38 2006 -- 000
+Dec 03 09:23:38 2006 -- 000
+Dec 03 09:23:38 2006 -- 000
+Jan 03 09:47:02 2007 -- 000
+EOF
+
+if test x"$BZIP2_CMD" != x""; then
+    $BZIP2_CMD -z -c "${srcdir}/logfile_syslog.1" > logfile_syslog.1.bz2
+
+    touch -t 200711030923 logfile_syslog.1.bz2
+    run_test ./drive_logfile -t -f syslog_log logfile_syslog.1.bz2
+
+    check_output "bzip2 file not loaded?" <<EOF
+Dec 03 09:23:38 2006 -- 000
+Dec 03 09:23:38 2006 -- 000
+Dec 03 09:23:38 2006 -- 000
+Jan 03 09:47:02 2007 -- 000
+EOF
+fi
+
+# The TCSH format converts to local time, so we need to specify a TZ
+export TZ="UTC"
 run_test ./drive_logfile -t -f tcsh_history ${srcdir}/logfile_tcsh_history.0
 
 check_output "TCSH timestamp interpreted incorrectly?" <<EOF
-Nov 02 09:59:26 2006 -- 000
-Nov 02 09:59:26 2006 -- 000
-Nov 02 09:59:45 2006 -- 000
-Nov 02 09:59:45 2006 -- 000
+Nov 02 17:59:26 2006 -- 000
+Nov 02 17:59:26 2006 -- 000
+Nov 02 17:59:45 2006 -- 000
+Nov 02 17:59:45 2006 -- 000
 EOF
 
 run_test ./drive_logfile -t -f access_log ${srcdir}/logfile_access_log.0
@@ -57,18 +85,19 @@ Jul 20 22:59:29 2009 -- 000
 Jul 20 22:59:29 2009 -- 000
 EOF
 
+touch -t 200711030923 ${srcdir}/logfile_strace_log.0
 run_test ./drive_logfile -t -f strace_log ${srcdir}/logfile_strace_log.0
 
 check_output "strace_log timestamp interpreted incorrectly?" <<EOF
-Dec 31 08:09:33 1979 -- 814
-Dec 31 08:09:33 1979 -- 815
-Dec 31 08:09:33 1979 -- 815
-Dec 31 08:09:33 1979 -- 815
-Dec 31 08:09:33 1979 -- 816
-Dec 31 08:09:33 1979 -- 816
-Dec 31 08:09:33 1979 -- 816
-Dec 31 08:09:33 1979 -- 816
-Dec 31 08:09:33 1979 -- 816
+Nov 03 08:09:33 2007 -- 814
+Nov 03 08:09:33 2007 -- 815
+Nov 03 08:09:33 2007 -- 815
+Nov 03 08:09:33 2007 -- 815
+Nov 03 08:09:33 2007 -- 816
+Nov 03 08:09:33 2007 -- 816
+Nov 03 08:09:33 2007 -- 816
+Nov 03 08:09:33 2007 -- 816
+Nov 03 08:09:33 2007 -- 816
 EOF
 
 ##
@@ -77,18 +106,18 @@ run_test ./drive_logfile -v -f syslog_log ${srcdir}/logfile_syslog.0
 
 check_output "Syslog level interpreted incorrectly?" <<EOF
 0x05
-0x00
+0x03
 0x05
-0x00
+0x03
 EOF
 
 run_test ./drive_logfile -v -f tcsh_history ${srcdir}/logfile_tcsh_history.0
 
 check_output "TCSH level interpreted incorrectly?" <<EOF
-0x40
-0xc0
-0x40
-0xc0
+0x03
+0x83
+0x03
+0x83
 EOF
 
 run_test ./drive_logfile -v -f access_log ${srcdir}/logfile_access_log.0
@@ -125,6 +154,7 @@ check_output "generic_log level interpreted incorrectly?" <<EOF
 0x02
 EOF
 
+touch -t 200711030923 ${srcdir}/logfile_glog.0
 run_test ./drive_logfile -t -f glog_log ${srcdir}/logfile_glog.0
 
 check_output "glog_log timestamp interpreted incorrectly?" <<EOF
